package com.soft.config;

import com.soft.enums.AuthEnum;
import com.soft.enums.CommEnum;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.web.bind.annotation.RequestMethod;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.builders.ResponseMessageBuilder;
import springfox.documentation.service.Contact;
import springfox.documentation.service.ResponseMessage;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Configuration
@EnableSwagger2
@Order(55)
public class SwaggerConfig extends WebSecurityConfigurerAdapter {

    @Value("http:localhost:${server.port:8088}")
    private String url;

    @Bean
    public Docket docketCommon() {
        List<ResponseMessage> responseMessageList = new ArrayList<>();
        Arrays.stream(CommEnum.values()).forEach(enums -> {
            responseMessageList.add(
                    new ResponseMessageBuilder().code(enums.getCode()).message(enums.getMessage()).build()
            );
        });
        Arrays.stream(AuthEnum.values()).forEach(enums -> {
            responseMessageList.add(
                    new ResponseMessageBuilder().code(enums.getCode()).message(enums.getMessage()).build()
            );
        });
        return new Docket(DocumentationType.SWAGGER_2)
                .globalResponseMessage(RequestMethod.GET, responseMessageList)
                .globalResponseMessage(RequestMethod.POST, responseMessageList)
                .globalResponseMessage(RequestMethod.PUT, responseMessageList)
                .globalResponseMessage(RequestMethod.DELETE, responseMessageList)
                .groupName("soft").select()
                .apis(RequestHandlerSelectors.basePackage("com.soft.controller"))
                .paths(PathSelectors.any())
                .build()
                .apiInfo(new ApiInfoBuilder().
                        title("soft API").
                        description("接口API").
                        contact(new Contact("", "", "")).termsOfServiceUrl(url).
                        version("1.0").
                        build());
    }

    @Override
    protected void configure(HttpSecurity http) throws Exception {
        http
        .authorizeRequests()
         // 限制 Swagger 路径需要 ADMIN 权限
        .antMatchers("/swagger-ui/**", "/v3/api-docs/**").hasRole("ADMIN")
         // 其他接口放行
        .anyRequest().permitAll()
        .and()
         // 启用 Basic 认证（浏览器弹窗登录）
         .httpBasic();
    }

}