package com.soft.controller;


import cn.hutool.core.collection.CollectionUtil;
import com.soft.annotations.HasPermission;
import com.soft.annotations.Valid;
import com.soft.bo.purchase.QueryPurchaseOrderBo;
import com.soft.dto.ApprovePassOrderDTO;
import com.soft.entitys.voice.Product;
import com.soft.entitys.voice.PurchaseOrderDetail;
import com.soft.entitys.voice.SaleOrder;
import com.soft.enums.OrderStatus;
import com.soft.resp.InvokeResult;
import com.soft.resp.InvokeResultBuilder;
import com.soft.service.ProductService;
import com.soft.service.PurchaseOrderDetailService;
import com.soft.service.PurchaseOrderService;
import com.soft.utils.PageResult;
import com.soft.utils.PageResultUtil;
import com.soft.utils.convert.purchase.QueryPurchaseOrderBOUtil;
import com.soft.vo.sale.QuerySaleOrderVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.soft.resp.Result;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/purchase/order")
@Api(tags = "采购订单")
public class PurchaseOrderController extends BaseController{

    @Autowired
    private PurchaseOrderService purchaseOrderService;

    @Autowired
    private PurchaseOrderDetailService purchaseOrderDetailService;

    @Autowired
    private ProductService productService;


    /**
     * 根据状态查询采购订单总数
     */
    @ApiOperation("根据状态查询采购订单总数")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "订单审核状态（0-待审核，3-审核通过，6-审核拒绝）", name = "status", paramType = "query")})
    @GetMapping("/total")
    public Result queryPurchaseTotal(
            Integer status) {
        if (OrderStatus.valueOfCode(status) == null){
            return Result.success(0);
        }
        Integer total = purchaseOrderService.getOrderSumByStatus(status);
        return Result.success(total);
    }

    /**
     * 采购订单审核
     */
    @ApiOperation("采购订单审核")
    @HasPermission(
            value = {"sale:order:approve"},
            userId = "#approveOrder.sysUserId" // 使用SpEL引用方法参数
    )
    @GetMapping("/approve")
    public Result approvePass(@RequestBody @Valid ApprovePassOrderDTO dto) {
        purchaseOrderService.approvePass(dto);
        return Result.success();

    }

    /**
     * 查询未审核采购订单列表
     */
    @ApiOperation("查询未审核采购订单列表")
    @GetMapping("/pendList")
    public InvokeResult<PageResult<QueryPurchaseOrderBo>> pendApproveList(QuerySaleOrderVo vo) {
        PageResult<SaleOrder> pageResult = purchaseOrderService.pendApproveList(getPageIndex(vo), getPageSize(vo), vo.getStatus());

        List<SaleOrder> datas = pageResult.getDatas();
        List<QueryPurchaseOrderBo> results = null;

        if (!CollectionUtil.isEmpty(datas)) {
            results = datas.stream().map(QueryPurchaseOrderBOUtil::toQueryPurchaseOrderBo).collect(Collectors.toList());
        }
        //商品名称处理
        if(results!=null){
            for (QueryPurchaseOrderBo result : results) {
                List<PurchaseOrderDetail> orderDetails=purchaseOrderDetailService.getByOrderId(result.getId());

                List<String> productNames = new ArrayList<>();
                for (PurchaseOrderDetail orderDetail : orderDetails) {
                    Product product= productService.findById(orderDetail.getProductId());
                    if (product != null && product.getName() != null) {
                        productNames.add(product.getName());
                    }

                }
                result.setProductName(String.join(", ", productNames));
            }
        }
        return InvokeResultBuilder.success(PageResultUtil.rebuild(pageResult, results));
    }
}
