package com.soft.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.soft.annotations.HasPermission;
import com.soft.annotations.Valid;
import com.soft.bo.sale.QuerySaleOrderBo;
import com.soft.dto.ApprovePassOrderDTO;
import com.soft.dto.SysUserDTO;
import com.soft.entitys.voice.Product;
import com.soft.entitys.voice.SaleOrder;
import com.soft.entitys.voice.SaleOrderDetail;
import com.soft.enums.OrderStatus;
import com.soft.resp.InvokeResult;
import com.soft.resp.InvokeResultBuilder;
import com.soft.service.ProductService;
import com.soft.service.SaleOrderDetailService;
import com.soft.service.SaleOrderService;
import com.soft.service.SysUserService;
import com.soft.utils.PageResult;
import com.soft.utils.PageResultUtil;
import com.soft.utils.convert.sale.QuerySaleOrderBOUtil;
import com.soft.vo.sale.QuerySaleOrderVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.soft.resp.Result;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Api(tags = "销售订单")
@Validated
@RestController
@RequestMapping("/sale/order")
public class SaleOrderController extends BaseController{

    @Autowired
    private SaleOrderService saleOrderService;

    @Autowired
    private SysUserService sysUserService;

    @Autowired
    private SaleOrderDetailService saleOrderDetailService;

    @Autowired
    private ProductService productService;

    /**
     * 根据状态查询销售订单总数
     */
    @ApiOperation("根据状态查询销售订单总数")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "订单审核状态（0-待审核，3-审核通过，6-审核拒绝）", name = "status", paramType = "query")})
    @GetMapping("/total")
    public Result querySaleTotal(
            Integer status) {
        if (OrderStatus.valueOfCode(status) == null){
            return Result.success(0);
        }
        Integer total = saleOrderService.getOrderSumByStatus(status);
        return Result.success(total);
    }


    /**
     * 销售订单审核
     */
    @ApiOperation("销售订单审核")
    @HasPermission(
            value = {"sale:order:approve"},  //判断用户是否具有审核权限
            userId = "#approveOrder.sysUserId" // 使用SpEL引用方法参数
    )
    @GetMapping("/approve")
    public Result approvePass(@RequestBody @Valid ApprovePassOrderDTO approveOrder) {
       //1. 查询用户id 关联的后台用户id; 3.判断订单是否存在，且状态是否是已审核状态
        SysUserDTO sysUserDTO = sysUserService.selectById(approveOrder.getSysUserId());
        if (sysUserDTO == null){
            return Result.fail("系统用户不存在，请检查后重试！");
        }
        // 审核 修改订单状态
        saleOrderService.approve(approveOrder.getOrderId(), approveOrder.getSysUserId());
        return null;
    }

    /**
     * 查询未审核销售订单
     */
    @ApiOperation("查询未审核销售订单列表")
    @GetMapping("/pendList")
    public InvokeResult<PageResult<QuerySaleOrderBo>> pendApproveList(QuerySaleOrderVo vo) {
        PageResult<SaleOrder> pageResult = saleOrderService.pendApproveList(getPageIndex(vo), getPageSize(vo), vo.getStatus());

        List<SaleOrder> datas = pageResult.getDatas();
        List<QuerySaleOrderBo> results = null;

        //转换
        if (!CollectionUtil.isEmpty(datas)) {
            results = datas.stream().map(QuerySaleOrderBOUtil::toQuerySaleOrderBo).collect(Collectors.toList());
        }
        //商品名称处理
        if(results!=null){
            for (QuerySaleOrderBo result : results) {
                List<SaleOrderDetail> orderDetails=saleOrderDetailService.getBySaleOrderId(result.getId());

                List<String> productNames = new ArrayList<>();
                for (SaleOrderDetail orderDetail : orderDetails) {
                    Product product= productService.findById(orderDetail.getProductId());
                    if (product != null && product.getName() != null) {
                        productNames.add(product.getName());
                    }

                }
                result.setProductName(String.join(", ", productNames));
            }
        }
        return InvokeResultBuilder.success(PageResultUtil.rebuild(pageResult, results));
    }



}
