package com.soft.controller;

import com.soft.dto.BindPhoneNumberDTO;
import com.soft.dto.WeChatLoginRequestDTO;
import com.soft.resp.Result;
import com.soft.entitys.user.SystemUser;
import com.soft.dto.SysUserDTO;
import com.soft.entitys.user.User;
import com.soft.service.SysUserService;
import com.soft.service.UserService;
import com.soft.vo.BindPhoneVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Map;
import java.util.Objects;

@RestController
@RequestMapping("/user")
@Api(tags = "用户api")
public class UserController {

    @Autowired
    private UserService userService;

    @Autowired
    private SysUserService sysUserService;


    @PostMapping("/wxLogin")
    @ApiOperation(value = "微信授权登录")
    public Result<Map<String, Object>> login(@RequestBody WeChatLoginRequestDTO weChatLoginRequestDTO) {

        if(weChatLoginRequestDTO==null){
            return null;
        }

        Map<String, Object> userInfoMap = userService.getUserInfoMap(weChatLoginRequestDTO);

        return Result.success(userInfoMap);

    }

    @PostMapping("/getPhoneNumber")
    @ApiOperation(value = "绑定手机号及上级")
    public Result<BindPhoneVo> getPhoneNumber(@RequestBody BindPhoneNumberDTO bindPhoneNumberDto){
        if(bindPhoneNumberDto==null){
            return null;
        }

        return Result.success(userService.getPhoneNumber(bindPhoneNumberDto.getCode(),bindPhoneNumberDto.getPid(),bindPhoneNumberDto.getUId(),bindPhoneNumberDto.getType()));
    }


    @PostMapping("/bindSysUser")
    @ApiOperation(value = "绑定后台系统用户")
    public Result<String> bindSysUser(@RequestBody SystemUser systemUser){
        User user = userService.queryByOpenId(systemUser.getOpenId());
        if (user == null ){
            return Result.fail("用户未登录，请先登录！");
        }
        String[] tmpArr = systemUser.getUsername().split("@");
        if (tmpArr.length <= 1) {
            throw new RuntimeException("用户名或密码错误！");
        }
        String tenantId = tmpArr[0];
        String username = tmpArr[1];
        systemUser.setUsername(username);

        SysUserDTO sysUserDTO = sysUserService.query(systemUser);
        if(sysUserDTO == null ){
            return Result.fail("绑定失败，系统用户不存在！");
        }
        // 查询系统用户是否已经被绑定
        User user1 = userService.queryBySysUserId(sysUserDTO.getId());
        if (Objects.nonNull(user1)){
            return Result.fail("系统用户已被绑定！");
        }

        User updateUser = new User();
        updateUser.setId(user.getId());
        updateUser.setSysUserId(sysUserDTO.getId());
        int update = userService.updateUserById(updateUser);
        if (update > 0 ){
            return Result.success("绑定成功！");
        }
        return Result.fail("绑定失败,请排查！");
    }

    @GetMapping("/query")
    @ApiOperation(value = "查询用户信息")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "小程序用户id", name = "id", paramType = "query")})
    public Result<User> bindSysUser(Integer id){
        User user = userService.getUserById(id);
        return Result.success(user);
    }

}
