package com.soft.controller;

import com.soft.dto.OrderProportionDto;
import com.soft.dto.StatisticsProductStockDto;
import com.soft.dto.VoiceQueryDto;
import com.soft.entitys.Page;
import com.soft.entitys.QueryParams;
import com.soft.resp.Result;
import com.soft.service.VoiceQueryService;
import com.soft.utils.UnsupportedQueryException;
import com.soft.utils.VoiceQueryParser;
import com.soft.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/voice")
@Api(tags = "语音api")
public class VoiceQueryController {


    @Autowired
    private VoiceQueryService voiceQueryService;


    private final VoiceQueryParser queryParser;

    public VoiceQueryController(VoiceQueryService voiceQueryService) {
        this.queryParser = new VoiceQueryParser();
        this.voiceQueryService = voiceQueryService;

    }

    @PostMapping("/handleVoiceQuery")
    @ApiOperation(value = "语音助手")
    public Result handleVoiceQuery(@RequestBody VoiceQueryDto voiceQueryDto) {
        // 1. 解析语音文本

        try {
            QueryParams params = queryParser.parse(voiceQueryDto.getText());

            // 2. 执行查询
            return Result.success(voiceQueryService.processQuery(params));
        }catch (UnsupportedQueryException e){
//            Map<String, Object> errorResponse = new HashMap<>();
//            errorResponse.put("error", "不支持的查询类型");
//            errorResponse.put("received", voiceQueryDto.getText());
//            errorResponse.put("message", "请尝试以下查询类型：今日采购订单金额、今日销售订单金额");

            return Result.fail("请尝试以下查询类型：今日采购订单、今日销售订单");

        }catch (Exception e) {
            // 其他异常返回500
            return Result.fail("其他错误，请稍后重试！");
        }

    }

    @GetMapping("/statisticsTodaysOrderData")
    @ApiOperation(value = "今日-销售与采购情况")
    public Result<OrderDataVo> statisticsTodaysOrderData() {

        OrderDataVo orderDataVo=voiceQueryService.statisticsTodaysOrderData();

        return Result.success(orderDataVo);

    }

    @GetMapping("/statisticsMonthOrderData")
    @ApiOperation(value = "近30日-销售与采购情况")
    public Result<OrderDataMonthVo> statisticsMonthOrderData() {

        OrderDataMonthVo orderDataVo=voiceQueryService.statisticsMonthOrderData();

        return Result.success(orderDataVo);

    }

    @GetMapping("/statisticsWeekOrderData")
    @ApiOperation(value = "近7日-销售与采购情况")
    public Result<OrderDataMonthVo> statisticsWeekOrderData() {

        OrderDataWeekVo orderDataVo=voiceQueryService.statisticsWeekOrderData();

        return Result.success(orderDataVo);

    }

    @GetMapping("/orderMonthContrast")
    @ApiOperation(value = "每月-订单对比分析")
    public Result<OrderMonthContrastVo> orderMonthContrast() {

        OrderMonthContrastVo orderContrastVo=voiceQueryService.orderMonthContrast();

        return Result.success(orderContrastVo);

    }

    @GetMapping("/orderTodaysContrast")
    @ApiOperation(value = "每日-订单对比分析")
    public Result<OrderTodaysContrastVo> orderTodaysContrast() {

        OrderTodaysContrastVo orderContrastVo=voiceQueryService.orderTodaysContrast();

        return Result.success(orderContrastVo);

    }

    @PostMapping("/orderProportion")
    @ApiOperation(value = "订单占比")
    public Result<OrderProportionVo> orderProportion(@RequestBody OrderProportionDto orderProportionDto) {

        OrderProportionVo orderProportionVo=voiceQueryService.orderProportion(orderProportionDto);

        return Result.success(orderProportionVo);

    }

    @GetMapping("/productProductStockProportion")
    @ApiOperation(value = "商品库存占比")
    public Result<List<ProductStockProportionVo>> productProductStockProportion() {

        List<ProductStockProportionVo> list=voiceQueryService.productProductStockProportion();

        return Result.success(list);

    }

    @GetMapping("/saleSalerMonthRank")
    @ApiOperation(value = "本月-销售人员排名")
    public Result<List<SaleSalerRankVo>> saleSalerMonthRank() {

        List<SaleSalerRankVo> list=voiceQueryService.saleSalerMonthRank();

        return Result.success(list);

    }

    @GetMapping("/saleSalerWeekRank")
    @ApiOperation(value = "本周-销售人员排名")
    public Result<List<SaleSalerRankVo>> saleSalerWeekRank() {

        List<SaleSalerRankVo> list=voiceQueryService.saleSalerWeekRank();

        return Result.success(list);

    }

    @GetMapping("/saleOrderGrossMargin")
    @ApiOperation(value = "最近12个月-销售毛利分析")
    public Result<SaleOrderGrossMarginVo> saleOrderGrossMargin() {

        SaleOrderGrossMarginVo saleOrderGrossMarginVo=voiceQueryService.saleOrderGrossMargin();

        return Result.success(saleOrderGrossMarginVo);

    }

    @GetMapping("/saleOrderLastYearProfit")
    @ApiOperation(value = "去年-销售毛利分析")
    public Result<SaleOrderLastYearProfitVo> saleOrderLastYearProfit() {

        SaleOrderLastYearProfitVo saleOrderLastYearProfitVo=voiceQueryService.saleOrderLastYearProfit();

        return Result.success(saleOrderLastYearProfitVo);

    }

    @GetMapping("/saleOrderCurrentYearAmount")
    @ApiOperation(value = "年度累计销售额，销售数量，销售成本，毛利率")
    public Result<SaleOrderCurrentYearAmountVo> saleOrderCurrentYearAmount() {

        SaleOrderCurrentYearAmountVo saleOrderCurrentYearAmountVo=voiceQueryService.saleOrderCurrentYearAmount();

        return Result.success(saleOrderCurrentYearAmountVo);

    }

    @PostMapping("/statisticsProductStock")
    @ApiOperation(value = "商品库存")
    public Result<Page<List<ProductStockDataVo>>> statisticsProductStock(@RequestBody StatisticsProductStockDto statisticsProductStockDto) {

        Page<List<ProductStockDataVo>> list=voiceQueryService.statisticsProductStock(statisticsProductStockDto);
        return Result.success(list);
    }


}
