package com.soft.resp;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializeConfig;
import com.soft.enums.CommEnum;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

@Data
@ApiModel(description = "统一返回")
public class Result<T> {

    @ApiModelProperty("数据")
    private T data;
    @ApiModelProperty("请求状态码")
    private int code;
    @ApiModelProperty("消息")
    private String message;

    private Result(int code, String message, T data) {
        this.code = code;
        this.message = message;
        this.data = data;
    }

    private Result(int code, String message) {
        this.code = code;
        this.message = message;
    }

    public static Result success() {
        CommEnum e= CommEnum.SUCCESS;
        int code=e.getCode();
        String message=e.getMessage();
        return new Result(code,message);
    }

    public static <T> Result success(T data) {
        CommEnum e= CommEnum.SUCCESS;
        int code=e.getCode();
        String message=e.getMessage();
        return new Result<T>(code,message,data);
    }

    public static Result fail(String message) {
        CommEnum e= CommEnum.FAIL;
        int code=e.getCode();
        return new Result(code,message);
    }

    public static Result fail() {
        CommEnum e= CommEnum.FAIL;
        int code=e.getCode();
        String message=e.getMessage();
        return new Result(code,message);
    }

    public static Result fail(Enum e){
        SerializeConfig config = new SerializeConfig();
        config.configEnumAsJavaBean(e.getClass());
        String json= JSON.toJSONString(e,config);
        JSONObject jsonObject=JSON.parseObject(json);
        int code=jsonObject.getIntValue("code");
        String message=jsonObject.getString("message");
        return new Result(code,message);
    }

    public static <T> Result fail(Enum e, T data){
        SerializeConfig config = new SerializeConfig();
        config.configEnumAsJavaBean(e.getClass());
        String json= JSON.toJSONString(e,config);
        JSONObject jsonObject=JSON.parseObject(json);
        int code=jsonObject.getIntValue("code");
        String message=jsonObject.getString("message");
        return new Result<T>(code,message,data);
    }


    public static Result isEmpty(){
        CommEnum e= CommEnum.EMPTY_PARAM;
        int code=e.getCode();
        String message=e.getMessage();
        return new Result(code,message);
    }

    public static Result isEmpty(String message){
        CommEnum e= CommEnum.EMPTY_PARAM;
        int code=e.getCode();
        return new Result(code,message);
    }

    public static Result error(){
        CommEnum e= CommEnum.ERROR;
        int code=e.getCode();
        return new Result(code,e.getMessage());
    }

    @Override
    public String toString(){
        return  JSON.toJSONString(this);
    }

}
