package com.soft.service.impl;

import com.soft.components.security.PermissionCalcType;
import com.soft.entitys.sys.SysUserRole;
import com.soft.mapper.SysMenuMapper;
import com.soft.mapper.SysRoleMenuMapper;
import com.soft.mapper.SysUserRoleMapper;
import com.soft.service.PermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Set;

@Service
public class PermissionServiceImpl implements PermissionService {

    @Autowired
    private SysUserRoleMapper sysUserRoleMapper;

    @Autowired
    private SysRoleMenuMapper sysRoleMenuMapper;

    @Autowired
    private SysMenuMapper sysMenuMapper;




    @Override
    public boolean hasPermission(String sysUserId, String[] value, PermissionCalcType calcType) {
        // 根据系统用户id查询具体角色，根据角色查询具体的权限菜单树
        SysUserRole sysUserRole = sysUserRoleMapper.selectById(sysUserId);
        //根据角色id查询菜单权限集合
        Set<String> rolePermissions = sysMenuMapper.getRolePermissionByUserId(sysUserRole.getRoleId());
        // 如果满足任意其中一个权限
        if (calcType == PermissionCalcType.OR) {
            // 任意一个权限匹配即可
            for (String permission : value) {
                if (rolePermissions.contains(permission)) {
                    return true;
                }
            }
            return false;
        } else if (calcType == PermissionCalcType.AND) {
            // 必须全部权限匹配
            for (String permission : value) {
                if (!rolePermissions.contains(permission)) {
                    return false;
                }
            }
            return true;
        } else {
            throw new IllegalArgumentException("不支持的权限匹配类型: " + calcType);
        }
    }
}
