package com.soft.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.pagehelper.PageInfo;
import com.soft.common.util.DateUtils;
import com.soft.dto.ApprovePassOrderDTO;
import com.soft.entitys.OrderPayType;
import com.soft.entitys.voice.PurchaseOrder;
import com.soft.entitys.voice.SaleOrder;
import com.soft.enums.OrderStatus;
import com.soft.events.order.impl.ApprovePassPurchaseOrderEvent;
import com.soft.mapper.PurchaseOrderMapper;
import com.soft.service.OrderPayTypeService;
import com.soft.service.PurchaseOrderService;
import com.soft.service.SaleOrderService;
import com.soft.utils.PageResult;
import com.soft.utils.PageResultUtil;
import org.apache.poi.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

@Service
public class PurchaseOrderServiceImpl implements PurchaseOrderService {

    @Autowired
    private PurchaseOrderMapper purchaseOrderMapper;

    @Autowired
    private SaleOrderService saleOrderService;

    @Autowired
    private OrderPayTypeService orderPayTypeService;

    @Autowired
    private ApplicationEventPublisher eventPublisher;

    @Override
    public Integer getOrderSumByStatus(Integer status) {
        return purchaseOrderMapper.getOrderSumByStatus(status);
    }

    @Override
    public PageResult<SaleOrder> pendApproveList(int pageIndex, int pageSize, Integer status) {
        int offset = (pageIndex - 1) * pageSize;
        Integer total = getOrderSumByStatus(status);
        //根据状态查询订单列表
        List<SaleOrder> datas = purchaseOrderMapper.selectListByStatus(pageSize, offset, status);
        PageResult<SaleOrder> convert = PageResultUtil.convert(new PageInfo<>(datas));
        convert.setTotalCount(total);
        return convert;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void approvePass(ApprovePassOrderDTO dto) {

        PurchaseOrder order = purchaseOrderMapper.selectById(dto.getOrderId());
        if (order == null) {
            throw new RuntimeException("订单不存在！");
        }

        if (order.getStatus() != OrderStatus.CREATED.getCode()
                && order.getStatus() != OrderStatus.APPROVE_REFUSE.getCode()) {

            if (order.getStatus() == OrderStatus.APPROVE_PASS.getCode()) {
                throw new RuntimeException("订单已审核通过，不允许继续执行审核！");
            }

            throw new RuntimeException("订单无法审核通过！");
        }

        if(order.getLinkNumber()!=null && order.getLinkNumber()!=""){
            SaleOrder saleOrder=saleOrderService.selectById(order.getLinkNumber());
            saleOrder.setPurchaseStatus(2);
            saleOrderService.updatePurchaseStatusbyId(saleOrder);
        }
        order.setStatus(OrderStatus.APPROVE_PASS.getCode());

        List<Integer> statusList = new ArrayList<>();
        statusList.add(OrderStatus.CREATED.getCode());
        statusList.add(OrderStatus.APPROVE_REFUSE.getCode());

        LambdaUpdateWrapper<PurchaseOrder> updateOrderWrapper = Wrappers.lambdaUpdate(
                PurchaseOrder.class)
                .set(PurchaseOrder::getApproveBy, dto.getSysUserId())
                .set(PurchaseOrder::getApproveTime, LocalDateTime.now())
                .eq(PurchaseOrder::getId, order.getId())
                .in(PurchaseOrder::getStatus, statusList);

        if (purchaseOrderMapper.update(order, updateOrderWrapper) != 1) {
            throw new RuntimeException("订单信息已过期，请刷新重试！");
        }

        if (order.getTotalAmount().compareTo(BigDecimal.ZERO) > 0) {
            List<OrderPayType> orderPayTypes = orderPayTypeService.findByOrderId(order.getId());
            if (CollectionUtil.isEmpty(orderPayTypes)) {
                throw new RuntimeException("单据没有约定支付，请检查！");
            }
        }

        this.sendApprovePassEvent(order);

    }
    private void sendApprovePassEvent(PurchaseOrder order) {

        ApprovePassPurchaseOrderEvent event = new ApprovePassPurchaseOrderEvent(this);
        event.setId(order.getId());
        event.setTotalAmount(order.getTotalAmount());
        event.setApproveTime(DateUtils.toLocalDateTime(order.getApproveTime()));
        eventPublisher.publishEvent(event);  // 直接调用 Spring 方法
    }
}
