package com.soft.service.impl;

import cn.hutool.core.codec.Base64;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.soft.dto.WeChatLoginRequestDTO;
import com.soft.entitys.user.User;
import com.soft.mapper.UserMapper;
import com.soft.service.UserService;
import com.soft.utils.ConstantUtil;
import com.soft.utils.HttpUtils;
import com.soft.utils.WxUtil;
import com.soft.vo.BindPhoneVo;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.math.BigDecimal;
import java.security.AlgorithmParameters;
import java.security.Security;
import java.util.*;


@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService {


    private static final Logger logger = LoggerFactory.getLogger(UserServiceImpl.class);

    ConstantUtil constantUtil = null;

    @Override
    public Map<String, Object> getUserInfoMap(WeChatLoginRequestDTO loginRequest) {
        Map<String, Object> map = new HashMap<>();
        String appId = constantUtil.WX_APPID;
        String secret = constantUtil.WX_SECRET;
        String url = "https://api.weixin.qq.com/sns/jscode2session?appid=" + appId + "&secret=" + secret + "&js_code=" + loginRequest.getCode() + "&grant_type=authorization_code";
        String resultStr = HttpUtils.sendGet(url);
        JSONObject SessionKeyOpenId = JSON.parseObject(resultStr);
        String openid = SessionKeyOpenId.getString("openid");
        String sessionKey = SessionKeyOpenId.getString("session_key");
        System.out.println("openid=" + openid + ",session_key=" + sessionKey);
        User user = super.baseMapper.getByOpenId(openid);

        JSONObject userInfo = decryptData(loginRequest.getEncryptedData(), sessionKey, loginRequest.getIv());

        if (user == null) {
            user = new User();
            String nickName = userInfo.getString("nickName");
            String avatarUrl = userInfo.getString("avatarUrl");
            user.setNickName(nickName);
            user.setAvatar(avatarUrl);
            user.setOpenId(openid);
            user.setCreateTime(new Date());
            super.baseMapper.insert(user);
        } else {
            //已存在
            user.setLastLoginTime(new Date());
            super.baseMapper.updateById(user);
            logger.info("用户openid已存在,不需要插入");
        }
        User userInf = super.baseMapper.getUserListByUid(user.getId());
        System.out.println(userInf);

        map.put("userInfo", userInf);
        map.put("sessionKey",sessionKey);
        return map;
    }

    @Override
    public JSONObject decryptData(String encryptedData, String sessionKey, String iv) {
        // 被加密的数据
        byte[] dataByte = Base64.decode(encryptedData);
        // 加密秘钥
        byte[] keyByte = Base64.decode(sessionKey);
        // 偏移量
        byte[] ivByte = Base64.decode(iv);
        try {
            // 如果密钥不足16位，那么就补足.这个if中的内容很重要
            int base = 16;
            if (keyByte.length % base != 0) {
                int groups = keyByte.length / base + 1;
                byte[] temp = new byte[groups * base];
                Arrays.fill(temp, (byte) 0);
                System.arraycopy(keyByte, 0, temp, 0, keyByte.length);
                keyByte = temp;
            }
            // 初始化
            Security.addProvider(new BouncyCastleProvider());
            Cipher cipher = Cipher.getInstance("AES/CBC/PKCS7Padding", "BC");
            SecretKeySpec spec = new SecretKeySpec(keyByte, "AES");
            AlgorithmParameters parameters = AlgorithmParameters.getInstance("AES");
            parameters.init(new IvParameterSpec(ivByte));
            cipher.init(Cipher.DECRYPT_MODE, spec, parameters);// 初始化
            byte[] resultByte = cipher.doFinal(dataByte);
            if (null != resultByte && resultByte.length > 0) {
                String result = new String(resultByte, "UTF-8");
                return JSONObject.parseObject(result);
            }
        } catch (Exception e) {
            logger.error("解密加密信息报错", e.getMessage());
        }
        return null;
    }

    @Override
    public BindPhoneVo getPhoneNumber(String code, Integer pid, Integer uId, Integer type) {

        String accessToken = WxUtil.getToken();
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("code", code);
        String msgBody = JSONObject.toJSONString(paramMap);// +
        String resultStr = HttpUtils.sendPost("https://api.weixin.qq.com/wxa/business/getuserphonenumber?access_token=" + accessToken,
                "application/json; charset=UTF-8", msgBody, 5000, 5000, null);
        System.out.println("resultStr ：" + resultStr);
        JSONObject resultJson = JSON.parseObject(resultStr);

        if (resultJson.getInteger("errcode") != 0) {
            System.out.println("null");
        }
        Object phoneNumber = resultJson.getJSONObject("phone_info").get("phoneNumber");
        BindPhoneVo bindPhoneVo = new BindPhoneVo();

        User user = super.baseMapper.getUserListByUid(uId);

        if (type == null && pid == null) {
            user.setPhone((String) phoneNumber);
        }

        if ((pid != null && type != null)) {
            if (type == 1) {
                user.setLevelId(1);
                bindPhoneVo.setLevel(1);
                user.setPid(pid);
                user.setSpreadTime(new Date());

                Integer count=super.baseMapper.getUpLevelUser(pid);

                User pUser=super.baseMapper.getUserListByUid(pid);
                if(pUser.getLevelId()==1){
                    if(count >=9){
                        QueryWrapper<User> queryWrapperUser = new QueryWrapper<>();
                        queryWrapperUser.eq("id", pid);
                        pUser.setLevelId(2);
                        super.baseMapper.update(pUser, queryWrapperUser);
                    }
                }

            }
        }

        user.setPhone((String) phoneNumber);
        QueryWrapper<User> queryWrapperUser = new QueryWrapper<>();
        queryWrapperUser.eq("id", user.getId());
        super.baseMapper.update(user, queryWrapperUser);
        bindPhoneVo.setPhoneNumber((String) phoneNumber);

        return bindPhoneVo;
    }

    @Override
    public User queryByOpenId(String openId) {
        return super.baseMapper.getByOpenId(openId);
    }

    @Override
    public int updateUserById(User user) {
        UpdateWrapper<User> updateWrapper = new UpdateWrapper<>();
        updateWrapper
                .eq("id", user.getId())
                .set("sys_user_id", user.getSysUserId());

        return super.baseMapper.update(user, updateWrapper);
    }

    @Override
    public User queryBySysUserId(String sysUserId) {
        QueryWrapper<User> queryWrapperUser = new QueryWrapper<>();
        queryWrapperUser.eq("sys_user_id", sysUserId);
        return super.baseMapper.selectOne(queryWrapperUser);

    }

    @Override
    public User getUserById(Integer userId) {
        QueryWrapper<User> queryWrapperUser = new QueryWrapper<>();
        queryWrapperUser.eq("id", userId);
        return super.baseMapper.selectOne(queryWrapperUser);
    }

}
