package com.soft.service.impl;

import com.soft.dto.OrderProportionDto;
import com.soft.dto.StatisticsProductStockDto;
import com.soft.dto.SysUserDTO;
import com.soft.entitys.Page;
import com.soft.entitys.QueryParams;
import com.soft.entitys.voice.*;
import com.soft.mapper.*;
import com.soft.service.VoiceQueryService;
import com.soft.utils.PieChartData;
import com.soft.utils.ProportionUtils;
import com.soft.utils.UnsupportedQueryException;
import com.soft.vo.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.Year;
import java.time.YearMonth;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class VoiceQueryServiceImpl implements VoiceQueryService {

    private static final Logger logger = LoggerFactory.getLogger(VoiceQueryServiceImpl.class);

    @Autowired
    private SaleOrderMapper saleOrderMapper;

    @Autowired
    private PurchaseOrderMapper purchaseOrderMapper;

    @Autowired
    private ProductMapper productMapper;

    @Autowired
    private PurchaseReturnMapper purchaseReturnMapper;

    @Autowired
    private PurchaseReturnDetailMapper purchaseReturnDetailMapper;

    @Autowired
    private PurchaseOrderDetailMapper purchaseOrderDetailMapper;

    @Autowired
    private CustomerMapper customerMapper;

    @Autowired
    private SaleOrderDetailMapper saleOrderDetailMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private ProductStockLogMapper productStockLogMapper;

    @Autowired
    private SaleOutSheetMapper saleOutSheetMapper;

    @Autowired
    private SaleOutSheetDetailMapper saleOutSheetDetailMapper;

    @Autowired
    private SettleCheckSheetMapper settleCheckSheetMapper;

    @Autowired
    private SettleCheckSheetDetailMapper settleCheckSheetDetailMapper;

    @Autowired
    private SettleSheetMapper settleSheetMapper;

    @Autowired
    private SettleSheetDetailMapper settleSheetDetailMapper;

    @Autowired
    private SupplierMapper supplierMapper;

    @Autowired
    private ProductStockMapper productStockMapper;

    @Autowired
    private StoreCenterMapper storeCenterMapper;

    @Override
    public Object processQuery(QueryParams params) {
        if (params.getType() == null) {
            throw new UnsupportedQueryException("未识别的查询类型");
        } else {
            switch (params.getType()) {
                case TODAY_PURCHASE_AMOUNT:
                    return getTodayPurchaseAmount();
                case TODAY_SALES_AMOUNT:
                    return getTodaySalesAmount();
                case PRODUCT_INVENTORY:
                    return QueryProductInventory(params.getProductName());
                case TODAY_PURCHASE_ORDERS:
                    return getTodayPurchaseOrder();
                case TODAY_PURCHASE_RETURN:
                    return getTodaysPurchaseOrderReturn();
                case CUSTOMER_NAME_SEARCH:
                    return getSaleOrderByCustomerName(params.getCustomerName());
                case USER_NAME_ORDERS_SEARCH:
                    return getSaleOrderByUserName(params.getUserName());
                case SALES_ORDERS_BY_DATE:
                    return selectSaleOrdersByDate(params.getQueryDate());
                case PURCHASE_ORDERS_BY_DATE:
                    return selectPurchaseOrdersByDate(params.getQueryDate());
                case PRODUCT_STOCK_LOG_BY_NAME:
                    return getProductStockLogs(params.getProductName());
                case TODAY_OUTBOUND_ORDERS:
                    return selectSaleOutSheetToDays();
                case REVENUE_COUNT:
                    return checkOrderAmountCount();
                case ACCOUNTS_PAYABLE:
                    return settleCheckAccountsPayable();
                case PAYMENT_TOF_FEES:
                    return settleSheetPaymentOfFees();
                case TOTAL_PROFIT:
                    return saleOrderTotalProfit();
                case MONTH_SALE_AMOUNT:
                    return getMonthSalesAmount();
                case MONTH_PURCHASE_AMOUNT:
                    return getMonthPurchaseAmount();
                case YEAR_SALE_AMOUNT:
                    return getYearSalesAmount();
                case YEAR_PURCHASE_AMOUNT:
                    return getYearPurchaseAmount();
                default:
                    throw new IllegalArgumentException("不支持的查询类型");
            }
        }

    }

    @Override
    public OrderDataVo statisticsTodaysOrderData() {


        OrderDataVo orderDataVo=new OrderDataVo();
        List<SaleOrderVo> saleOrderVoList= saleOrderMapper.CheckTodysSaleOrderLits();

        BigDecimal todayTotalSaleAmount = BigDecimal.ZERO;

        for (SaleOrderVo saleOrderVo : saleOrderVoList) {
            if (saleOrderVo.getTotalAmount() != null) {
                todayTotalSaleAmount = todayTotalSaleAmount.add(saleOrderVo.getTotalAmount());
            }
        }

        orderDataVo.setTodaySaleOrderAmount(todayTotalSaleAmount);
        orderDataVo.setTodaySaleQuantityCount(saleOrderVoList.size());


        BigDecimal todayTotalPurchaseAmount = BigDecimal.ZERO;
        List<PurchaseOrderVo> purchaseOrderVoList=purchaseOrderMapper.TodaysPurchaseOrder();
        for (PurchaseOrderVo purchaseOrderVo : purchaseOrderVoList) {
            if (purchaseOrderVo.getTotalAmount() != null) {
                todayTotalPurchaseAmount = todayTotalPurchaseAmount.add(purchaseOrderVo.getTotalAmount());
            }
        }
        orderDataVo.setTodayPurchaseOrderAmount(todayTotalPurchaseAmount);
        orderDataVo.setTodayPurchaseQuantityCount(purchaseOrderVoList.size());





        return orderDataVo;
    }

    @Override
    public OrderDataMonthVo statisticsMonthOrderData() {

        OrderDataMonthVo orderDataMonthVo=new OrderDataMonthVo();
        BigDecimal thisMonthTotalPurchaseAmount = BigDecimal.ZERO;
        List<PurchaseOrderVo> thisMonthPurchaseList=purchaseOrderMapper.thisMonthPurchaseOrder();

        for (PurchaseOrderVo purchaseOrderVo : thisMonthPurchaseList) {
            if (purchaseOrderVo.getTotalAmount() != null) {
                thisMonthTotalPurchaseAmount = thisMonthTotalPurchaseAmount.add(purchaseOrderVo.getTotalAmount());
            }
        }

        orderDataMonthVo.setThisMonthPurchaseOrderAmount(thisMonthTotalPurchaseAmount);
        orderDataMonthVo.setThisMonthPurchaseQuantityCount(thisMonthPurchaseList.size());


        BigDecimal thisMonthTotalSaleAmount = BigDecimal.ZERO;
        List<SaleOrderVo> thisMonthSaleList=saleOrderMapper.CheckThisMonthSaleOrderList();
        for (SaleOrderVo saleOrderVo : thisMonthSaleList) {
            if (saleOrderVo.getTotalAmount() != null) {
                thisMonthTotalSaleAmount = thisMonthTotalSaleAmount.add(saleOrderVo.getTotalAmount());
            }
        }

        orderDataMonthVo.setThisMonthSaleOrderAmount(thisMonthTotalSaleAmount);
        orderDataMonthVo.setThisMonthSaleQuantityCount(thisMonthSaleList.size());

        return orderDataMonthVo;
    }

    @Override
    public OrderDataWeekVo statisticsWeekOrderData() {

        OrderDataWeekVo orderDataWeekVo=new OrderDataWeekVo();
        BigDecimal thisMonthTotalPurchaseAmount = BigDecimal.ZERO;
        List<PurchaseOrderVo> thisMonthPurchaseList=purchaseOrderMapper.thisWeekPurchaseOrder();

        for (PurchaseOrderVo purchaseOrderVo : thisMonthPurchaseList) {
            if (purchaseOrderVo.getTotalAmount() != null) {
                thisMonthTotalPurchaseAmount = thisMonthTotalPurchaseAmount.add(purchaseOrderVo.getTotalAmount());
            }
        }

        orderDataWeekVo.setThisWeekPurchaseOrderAmount(thisMonthTotalPurchaseAmount);
        orderDataWeekVo.setThisWeekPurchaseQuantityCount(thisMonthPurchaseList.size());


        BigDecimal thisMonthTotalSaleAmount = BigDecimal.ZERO;
        List<SaleOrderVo> thisMonthSaleList=saleOrderMapper.CheckThisWeekSaleOrderList();
        for (SaleOrderVo saleOrderVo : thisMonthSaleList) {
            if (saleOrderVo.getTotalAmount() != null) {
                thisMonthTotalSaleAmount = thisMonthTotalSaleAmount.add(saleOrderVo.getTotalAmount());
            }
        }

        orderDataWeekVo.setThisWeekSaleOrderAmount(thisMonthTotalSaleAmount);
        orderDataWeekVo.setThisWeekSaleQuantityCount(thisMonthSaleList.size());

        return orderDataWeekVo;
    }

    @Override
    public OrderMonthContrastVo orderMonthContrast() {

        OrderMonthContrastVo orderMonthContrastVo=new OrderMonthContrastVo();

        // 生成最近12个月的月份标签
        LocalDate now = LocalDate.now();
        List<String> months = new ArrayList<>();
        List<YearMonth> yearMonths = new ArrayList<>();

        // 倒序生成月份列表（从最早月份到当前月）
        for (int i = 11; i >= 0; i--) {
            YearMonth ym = YearMonth.from(now.minusMonths(i));
            yearMonths.add(ym);
            months.add(ym.getMonthValue() + "月");
        }

        // 初始化金额列表（12个0）
        List<BigDecimal> saleAmounts = ProportionUtils.initZeroList(12);
        List<BigDecimal> purchaseAmounts = ProportionUtils.initZeroList(12);

        // 创建月份索引映射表
        Map<YearMonth, Integer> indexMap = new HashMap<>();
        for (int i = 0; i < yearMonths.size(); i++) {
            indexMap.put(yearMonths.get(i), i);
        }

        // 处理销售数据
        List<Map<String, Object>> sales = saleOrderMapper.orderMonthContrastSaleOrder();
        ProportionUtils.fillAmounts(sales, saleAmounts, indexMap, "sale_amount");

        // 处理采购数据
        List<Map<String, Object>> purchases = purchaseOrderMapper.orderMonthContrastPurchaseOrder();
        ProportionUtils.fillAmounts(purchases, purchaseAmounts, indexMap, "purchase_amount");

        orderMonthContrastVo.setXAxisData(months);
        orderMonthContrastVo.setSaleAmounts(saleAmounts);
        orderMonthContrastVo.setPurchaseAmounts(purchaseAmounts);
        orderMonthContrastVo.setChartType("chart");  // 标识为按月统

        return orderMonthContrastVo;
    }

    @Override
    public OrderTodaysContrastVo orderTodaysContrast() {
        OrderTodaysContrastVo vo = new OrderTodaysContrastVo();

        // 生成最近30天的日期标签
        LocalDate now = LocalDate.now();
        List<String> xAxisData = new ArrayList<>();
        for (int i = 30; i > 0; i--) { // 生成从30天前到昨天的日期（共30天）
            LocalDate date = now.minusDays(i - 1); // i-1确保包含当天
            xAxisData.add(date.getMonthValue() + "/" + date.getDayOfMonth());
        }

        // 初始化金额列表
        List<BigDecimal> saleAmounts = new ArrayList<>(Collections.nCopies(30, BigDecimal.ZERO));
        List<BigDecimal> purchaseAmounts = new ArrayList<>(Collections.nCopies(30, BigDecimal.ZERO));

        // 获取销售数据
        List<Map<String, Object>> sales = saleOrderMapper.orderTodaysContrastSaleOrder();
        // 获取采购数据
        List<Map<String, Object>> purchases = purchaseOrderMapper.orderTodaysContrastPurchaseOrder();

        ProportionUtils.dayFillAmounts(purchases, purchaseAmounts, now, "purchase_amount"); // 采购用purchase_amount
        ProportionUtils.dayFillAmounts(sales, saleAmounts, now, "sale_amount"); // 销售用sale_amount

        vo.setXAxisData(xAxisData);
        vo.setSaleAmounts(saleAmounts);
        vo.setPurchaseAmounts(purchaseAmounts);
        vo.setChartType("chart");
        return vo;
    }

    @Override
    public OrderProportionVo orderProportion(OrderProportionDto orderProportionDto) {

        if (orderProportionDto.getMonth() == null || orderProportionDto.getMonth() < 1 || orderProportionDto.getMonth() > 12) {
            throw new IllegalArgumentException("Invalid month");
        }

        //默认年份为当前年
        Integer currentYear = LocalDate.now().getYear();

        //查询采购金额
        BigDecimal purchaseAmount = purchaseOrderMapper.getPurchaseAmountByMonth(currentYear, orderProportionDto.getMonth());

        //查询销售金额
        BigDecimal saleAmount = saleOrderMapper.getSaleAmountByMonth(currentYear, orderProportionDto.getMonth());

        //计算采销占比
        String proportion = ProportionUtils.calculateProportion(purchaseAmount, saleAmount);


        OrderProportionVo vo = new OrderProportionVo();
        vo.setPurchaseAmount(purchaseAmount);
        vo.setSaleAmount(saleAmount);
        vo.setProportion(proportion);
        return vo;
    }

    @Override
    public List<ProductStockProportionVo> productProductStockProportion() {

        //获取原始数据并计算总库存
        List<ProductStockDataVo> dataList = productStockMapper.getProductStockProportion();
        int totalStock = dataList.stream()
                .mapToInt(ProductStockDataVo::getStockNum)
                .sum();

        BigDecimal total = totalStock == 0 ? BigDecimal.ONE : new BigDecimal(totalStock);

        //按库存降序排序
        List<ProductStockDataVo> sortedList = dataList.stream()
                .sorted((d1, d2) -> Integer.compare(d2.getStockNum(), d1.getStockNum()))
                .collect(Collectors.toList());

        //分离前十和剩余商品
        List<ProductStockDataVo> top10 = sortedList.stream()
                .limit(10)
                .collect(Collectors.toList());

        List<ProductStockDataVo> others = sortedList.stream()
                .skip(10)
                .collect(Collectors.toList());

        List<ProductStockProportionVo> result = new ArrayList<>();

        //添加总库存条目
        ProductStockProportionVo totalVo = new ProductStockProportionVo();
        totalVo.setProductName("总库存");
        totalVo.setStock(String.valueOf(totalStock));
        totalVo.setProportion(""); // 总库存不显示百分比
        result.add(totalVo);

        //处理前十商品（计算百分比）
        top10.forEach(data -> {
            ProductStockProportionVo vo = new ProductStockProportionVo();
            vo.setProductName(data.getProductName());
            vo.setStock(String.valueOf(data.getStockNum()));

            // 计算百分比
            BigDecimal ratio = new BigDecimal(data.getStockNum())
                    .divide(total, 4, RoundingMode.HALF_UP)
                    .multiply(new BigDecimal(100));
            vo.setProportion(String.format("%.2f%%", ratio));

            result.add(vo);
        });

        //处理其他商品（合并并计算百分比）
        if (!others.isEmpty()) {
            int othersTotal = others.stream()
                    .mapToInt(ProductStockDataVo::getStockNum)
                    .sum();

            ProductStockProportionVo othersVo = new ProductStockProportionVo();
            othersVo.setProductName("其他商品");
            othersVo.setStock(String.valueOf(othersTotal));

            // 计算百分比
            BigDecimal othersRatio = new BigDecimal(othersTotal)
                    .divide(total, 4, RoundingMode.HALF_UP)
                    .multiply(new BigDecimal(100));
            othersVo.setProportion(String.format("%.2f%%", othersRatio));

            result.add(othersVo);
        }

        return result;
    }

    @Override
    public List<SaleSalerRankVo> saleSalerMonthRank() {
        //询本月所有销售订单
        List<SaleOrderVo> orders = saleOrderMapper.saleSalerMonthRank();

        Set<String> salerIds = orders.stream()
                .filter(order -> order.getSalerId() != null)
                .map(SaleOrderVo::getSalerId)
                .collect(Collectors.toSet());

        Map<String, String> salerNameMap = sysUserMapper.selectUsersByIds(salerIds).stream()
                .collect(Collectors.toMap(SysUserDTO::getId, SysUserDTO::getName));

        Map<String, BigDecimal> salerAmountMap = new HashMap<>();
        for (SaleOrderVo order : orders) {
            if (order.getSalerId() == null || order.getTotalAmount() == null) continue;

            String salerName = salerNameMap.getOrDefault(order.getSalerId(), "未知销售员");

            // 获取订单金额（空值处理）
            BigDecimal amount = Optional.ofNullable(order.getTotalAmount()).orElse(BigDecimal.ZERO);

            salerAmountMap.merge(salerName, amount, BigDecimal::add);
        }

        return salerAmountMap.entrySet().stream()
                .map(entry -> new SaleSalerRankVo(
                        entry.getKey(),
                        Optional.ofNullable(entry.getValue()).orElse(BigDecimal.ZERO)
                ))
                .sorted((a, b) -> {
                    BigDecimal aAmount = Optional.ofNullable(a.getSaleAmount()).orElse(BigDecimal.ZERO);
                    BigDecimal bAmount = Optional.ofNullable(b.getSaleAmount()).orElse(BigDecimal.ZERO);
                    return bAmount.compareTo(aAmount);
                })
                .collect(Collectors.toList());
    }

    @Override
    public List<SaleSalerRankVo> saleSalerWeekRank() {
        List<SaleOrderVo> orders = saleOrderMapper.saleSalerWeekRank();

        Set<String> salerIds = orders.stream()
                .filter(order -> order.getSalerId() != null)
                .map(SaleOrderVo::getSalerId)
                .collect(Collectors.toSet());

        Map<String, String> salerNameMap = sysUserMapper.selectUsersByIds(salerIds).stream()
                .collect(Collectors.toMap(SysUserDTO::getId, SysUserDTO::getName));

        Map<String, BigDecimal> salerAmountMap = new HashMap<>();
        for (SaleOrderVo order : orders) {
            if (order.getSalerId() == null || order.getTotalAmount() == null) continue;

            String salerName = salerNameMap.getOrDefault(order.getSalerId(), "未知销售员");

            // 获取订单金额（空值处理）
            BigDecimal amount = Optional.ofNullable(order.getTotalAmount()).orElse(BigDecimal.ZERO);

            salerAmountMap.merge(salerName, amount, BigDecimal::add);
        }

        return salerAmountMap.entrySet().stream()
                .map(entry -> new SaleSalerRankVo(
                        entry.getKey(),
                        Optional.ofNullable(entry.getValue()).orElse(BigDecimal.ZERO)
                ))
                .sorted((a, b) -> {
                    BigDecimal aAmount = Optional.ofNullable(a.getSaleAmount()).orElse(BigDecimal.ZERO);
                    BigDecimal bAmount = Optional.ofNullable(b.getSaleAmount()).orElse(BigDecimal.ZERO);
                    return bAmount.compareTo(aAmount);
                })
                .collect(Collectors.toList());
    }

    @Override
    public SaleOrderGrossMarginVo saleOrderGrossMargin() {

        // 获取最近12个月的订单数据
        List<SaleOrderVo>  list=saleOrderMapper.latelyDecemberSaleOrderProfit();

        LocalDate currentDate = LocalDate.now();
        List<String> months = new ArrayList<>();
        List<YearMonth> yearMonths = new ArrayList<>();
        for (int i = 11; i >= 0; i--) {
            YearMonth yearMonth = YearMonth.from(currentDate.minusMonths(i));
            yearMonths.add(yearMonth);
            months.add(yearMonth.getMonthValue() + "月");
        }
        List<BigDecimal> profits = yearMonths.stream()
                .map(ym -> BigDecimal.ZERO)
                .collect(Collectors.toCollection(ArrayList::new));

        Map<YearMonth, Integer> monthIndexMap = new HashMap<>();
        for (int i = 0; i < yearMonths.size(); i++) {
            monthIndexMap.put(yearMonths.get(i), i);
        }

        // 计算每个月的毛利总和
        for (SaleOrderVo order : list) {
            // 转换时间到年月维度
            LocalDate orderDate = order.getCreateTime().toInstant()
                    .atZone(ZoneId.systemDefault())
                    .toLocalDate();
            YearMonth orderYearMonth = YearMonth.from(orderDate);

            // 找到对应的月份索引
            Integer index = monthIndexMap.get(orderYearMonth);
            if (index != null) {
                BigDecimal profit = order.getThisOrderProfit() != null ?
                        order.getThisOrderProfit() : BigDecimal.ZERO;
                profits.set(index, profits.get(index).add(profit));
            }
        }

        for (SaleOrderVo saleOrderVo : list) {
            saleOrderVo.getThisOrderProfit();
        }

        SaleOrderGrossMarginVo result = new SaleOrderGrossMarginVo();
        result.setChartType("chart");
        result.setXAxisData(months);
        result.setSaleAmounts(profits);

        return result;
    }

    @Override
    public SaleOrderLastYearProfitVo saleOrderLastYearProfit() {

        List<SaleOrderVo> orders = saleOrderMapper.lastYearSaleOrderProfit();

        LocalDate currentDate = LocalDate.now();
        Year lastYear = Year.from(currentDate.minusYears(1));
        List<String> months = new ArrayList<>();
        for (int month = 1; month <= 12; month++) {
            months.add(String.valueOf(month) + "月");
        }

        List<BigDecimal> profits = months.stream()
                .map(month -> BigDecimal.ZERO)
                .collect(Collectors.toCollection(ArrayList::new));

        Map<Integer, Integer> monthIndexMap = new HashMap<>();
        for (int i = 0; i < 12; i++) {
            monthIndexMap.put(i + 1, i);
        }

        for (SaleOrderVo order : orders) {
            // 转换时间到月份维度
            LocalDate orderDate = order.getCreateTime().toInstant()
                    .atZone(ZoneId.systemDefault())
                    .toLocalDate();
            int orderMonth = orderDate.getMonthValue();

            // 找到对应的月份索引
            Integer index = monthIndexMap.get(orderMonth);
            if (index != null) {
                BigDecimal profit = order.getThisOrderProfit() != null ?
                        order.getThisOrderProfit() : BigDecimal.ZERO;
                profits.set(index, profits.get(index).add(profit));
            }
        }

        // 构建返回对象
        SaleOrderLastYearProfitVo result = new SaleOrderLastYearProfitVo();
        result.setChartType("chart");
        result.setXAxisData(months);
        result.setSaleAmounts(profits);

        return result;
    }

    @Override
    public SaleOrderCurrentYearAmountVo saleOrderCurrentYearAmount() {

        SaleOrderCurrentYearAmountVo vo = new SaleOrderCurrentYearAmountVo();

        // 获取数据：包含今年和去年（可能为0）
        List<Map<String, Object>> data = saleOrderMapper.getYearlyComparisonData();

        // 提取今年和去年的数据
        BigDecimal currentYearAmount = BigDecimal.ZERO;
        BigDecimal currentYearCost = BigDecimal.ZERO;
        BigDecimal currentYearSales = BigDecimal.ZERO;
        BigDecimal lastYearAmount = BigDecimal.ZERO;
        BigDecimal lastYearCost = BigDecimal.ZERO;
        BigDecimal lastYearSales = BigDecimal.ZERO;

        for (Map<String, Object> item : data) {
            Long yearLong = (Long) item.get("year");
            if (yearLong != null) {
                Integer year = yearLong.intValue(); // 安全地转换为 Integer
                // 使用 year
                BigDecimal amount = (BigDecimal) item.get("total_amount");
                BigDecimal sales = new BigDecimal(item.get("total_num").toString());
                BigDecimal cost = (BigDecimal) item.get("total_purchase_price");

                if (year == Year.now().getValue()) { // 今年
                    currentYearAmount = amount != null ? amount : BigDecimal.ZERO;
                    currentYearSales = sales;
                    currentYearCost = cost != null ? cost : BigDecimal.ZERO;
                } else { // 去年
                    lastYearAmount = amount != null ? amount : BigDecimal.ZERO;
                    lastYearSales = sales;
                    lastYearCost = cost != null ? cost : BigDecimal.ZERO;
                }
            }

        }

        // 填充基础数据
        vo.setCurrentYearAmount(currentYearAmount);
        vo.setSaleCount(currentYearSales.toPlainString());
        vo.setTotalPurchasePrice(currentYearCost);

        // 计算毛利率（百分比）
        if (currentYearAmount.compareTo(BigDecimal.ZERO) != 0) {
            BigDecimal grossMargin = currentYearAmount.subtract(currentYearCost)
                    .divide(currentYearAmount, 4, RoundingMode.HALF_UP)
                    .multiply(BigDecimal.valueOf(100));
            vo.setGrossMarginPercent(String.format("%.2f%%", grossMargin));
        } else {
            vo.setGrossMarginPercent("0.00%");
        }

        // 计算销售额同比增长率
        vo.setAmountGrowthPercent(ProportionUtils.calculateGrowth(currentYearAmount, lastYearAmount));

        // 计算销售数量同比增长率
        vo.setSaleCountGrowthPercent(ProportionUtils.calculateGrowth(currentYearSales, lastYearSales));

        // 计算成本同比增长率
        vo.setPurchasePriceGrowthPercent(ProportionUtils.calculateGrowth(currentYearCost, lastYearCost));

        // 计算毛利率同比增长率（需要计算去年毛利率）
        BigDecimal lastYearGrossMargin = BigDecimal.ZERO;
        if (lastYearAmount.compareTo(BigDecimal.ZERO) != 0) {
            lastYearGrossMargin = lastYearAmount.subtract(lastYearCost)
                    .divide(lastYearAmount, 4, RoundingMode.HALF_UP)
                    .multiply(BigDecimal.valueOf(100));
        }
        BigDecimal currentYearGrossMargin = new BigDecimal(vo.getGrossMarginPercent().replace("%", ""));
        vo.setGrossMarginGrowthPercent(ProportionUtils.calculateGrowth(currentYearGrossMargin, lastYearGrossMargin));

        return vo;

    }

    @Override
    public Page<List<ProductStockDataVo>> statisticsProductStock(StatisticsProductStockDto statisticsProductStockDto) {


        Integer count=productStockMapper.getProductStockListCount();
        Integer startNum=(statisticsProductStockDto.getPageNum()-1)*statisticsProductStockDto.getPageSize();


        List<ProductStockDataVo> list=productStockMapper.getProductStockList(startNum,statisticsProductStockDto.getPageSize());



        for (ProductStockDataVo productStockDataVo : list) {
            Product product=productMapper.getProductById(productStockDataVo.getProductId());
            productStockDataVo.setProductName(product.getName());

            StoreCenter storeCenter=storeCenterMapper.getStoreCenterById(productStockDataVo.getScId());
            productStockDataVo.setScName(storeCenter.getName());
        }

        return new Page<>(statisticsProductStockDto.getPageNum(),count,statisticsProductStockDto.getPageSize(),list);
    }


    private CheckTodaysPurchaseAmountVo getTodayPurchaseAmount() {

        CheckTodaysPurchaseAmountVo checkTodaysPurchaseAmountVo = new CheckTodaysPurchaseAmountVo();
        checkTodaysPurchaseAmountVo.setChartType("column");
        BigDecimal amount = purchaseOrderMapper.CheckTodaysPurchaseAmount();


        checkTodaysPurchaseAmountVo.setPurchaseAmount(amount);

        List<PurchaseOrderVo> list = purchaseOrderMapper.TodaysPurchaseOrder();
        Map<String, BigDecimal> supplierAmountMap = new HashMap<>();

        for (PurchaseOrderVo purchaseOrderVo : list) {
            Supplier supplier = supplierMapper.getSupplierById(purchaseOrderVo.getSupplierId());
            String supplierName = supplier.getName();
            purchaseOrderVo.setSupplierName(supplierName);
            BigDecimal totalAmount = purchaseOrderVo.getTotalAmount();

            supplierAmountMap.merge(
                    supplierName,
                    totalAmount,
                    BigDecimal::add  // 相同供应商的金额相加
            );

        }

        List<String> xAxisData = new ArrayList<>(supplierAmountMap.keySet()); // 横轴
        List<BigDecimal> yAxisData = new ArrayList<>(supplierAmountMap.values()); // 纵轴

        LocalDate currentDate = LocalDate.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String formattedDate = currentDate.format(formatter);
        DecimalFormat currencyFormat = new DecimalFormat("￥####.00");

        String formattedTotalAmount=null;
        if(saleOrderMapper.CheckTodaysOrderAmount()!=null){
            formattedTotalAmount = currencyFormat.format(amount);
        }else{
            formattedTotalAmount="0";
        }

        String msg = "好的，" + formattedDate + "，采购订单金额为: " + formattedTotalAmount + "，有其他需求可以随时告诉我哦！";


        checkTodaysPurchaseAmountVo.setMsg(msg);

        checkTodaysPurchaseAmountVo.setXAxisData(xAxisData);
        checkTodaysPurchaseAmountVo.setYAxisData(yAxisData);

        return checkTodaysPurchaseAmountVo;
    }


    private CheckTodysSalesAmountVo getTodaySalesAmount() {
        CheckTodysSalesAmountVo checkTodysSalesAmountVo = new CheckTodysSalesAmountVo();
        checkTodysSalesAmountVo.setChartType("column");
        checkTodysSalesAmountVo.setSalesAmount(saleOrderMapper.CheckTodaysOrderAmount());

        List<SaleOrderVo> list = saleOrderMapper.CheckTodysSaleOrderLits();

        Map<String, BigDecimal> supplierAmountMap = new HashMap<>();
        for (SaleOrderVo saleOrderVo : list) {
            if (saleOrderVo != null) {
                if (saleOrderVo.getSalerId() != null) {
                    SysUserDTO sysUserDTO = sysUserMapper.getSysUserById(saleOrderVo.getSalerId());
                    String name = sysUserDTO.getName();
                    saleOrderVo.setSalerName(name);
                    BigDecimal totalAmount = saleOrderVo.getTotalAmount();

                    supplierAmountMap.merge(
                            name,
                            totalAmount,
                            BigDecimal::add  // 相同供应商的金额相加
                    );
                }

            }

        }

        List<String> xAxisData = new ArrayList<>(supplierAmountMap.keySet()); // 横轴
        List<BigDecimal> yAxisData = new ArrayList<>(supplierAmountMap.values()); // 纵轴

        LocalDate currentDate = LocalDate.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String formattedDate = currentDate.format(formatter);

        DecimalFormat currencyFormat = new DecimalFormat("￥####.00");

        String formattedTotalAmount=null;
        if(saleOrderMapper.CheckTodaysOrderAmount()!=null){
            formattedTotalAmount = currencyFormat.format(saleOrderMapper.CheckTodaysOrderAmount());
        }else{
            formattedTotalAmount="0";
        }


        String msg = "好的，" + formattedDate + "，销售订单金额为: " + formattedTotalAmount + "，有其他需求可以随时告诉我哦！";
        checkTodysSalesAmountVo.setMsg(msg);

        checkTodysSalesAmountVo.setXAxisData(xAxisData);
        checkTodysSalesAmountVo.setYAxisData(yAxisData);

        return checkTodysSalesAmountVo;
    }

    private QueryProductInventoryVo QueryProductInventory(String productName) {
        // 1. 查询匹配的商品列表（如所有"联想"商品）
        List<ProductVo> matchedProducts = productMapper.QueryProductInventory(productName);

        // 2. 获取全库总库存（包括匹配和非匹配商品）
        Integer totalStock = productStockMapper.getProductStockNum();

        // 3. 计算匹配商品的总库存
        int matchedStock = matchedProducts.stream()
                .mapToInt(ProductVo::getStockNum)
                .sum();

        // 4. 计算其他商品库存（全库 - 匹配商品）
        int otherStock = totalStock - matchedStock;
        double otherPercent = (double) otherStock / totalStock * 100;

        // 5. 构建饼图数据（匹配商品 + 其他商品）
        List<PieChartData> pieChartData = new ArrayList<>();

        // 5.1 添加匹配的各个商品
        matchedProducts.forEach(product -> {
            double percent = (double) product.getStockNum() / totalStock * 100;
            pieChartData.add(new PieChartData(
                    product.getName(),
                    product.getStockNum(),
                    String.format("%.2f%%", percent)
            ));
        });

        // 5.2 添加"其他商品"选项
        pieChartData.add(new PieChartData(
                "其他商品",
                otherStock,
                String.format("%.2f%%", otherPercent)
        ));

        // 6. 构建消息文本
        StringBuilder msg = new StringBuilder("查询到的商品库存信息：\n");
        matchedProducts.forEach(product ->
                msg.append("商品：")
                        .append(product.getName())
                        .append("，剩余库存：")
                        .append(product.getStockNum())
                        .append("\n")
        );

        // 7. 返回结果
        QueryProductInventoryVo vo = new QueryProductInventoryVo();
        vo.setChartType("ring");
        vo.setPieChartData(pieChartData);
        vo.setMsg(msg.toString());
        return vo;
    }


    private PurchaseOrderChartVo getTodayPurchaseOrder() {

        PurchaseOrderChartVo purchaseOrderChartVo = new PurchaseOrderChartVo();
        List<PurchaseOrderVo> list = purchaseOrderMapper.TodaysPurchaseOrder();

//        List<String> xAxisSupplierNames = new ArrayList<>();
//        List<BigDecimal> yAxisOrderAmounts = new ArrayList<>();
        Map<String, BigDecimal> supplierAmountMap = new HashMap<>();

        for (PurchaseOrderVo purchaseOrderVo : list) {

            purchaseOrderVo.setPurchaseOrderDetailList(purchaseOrderDetailMapper.getPurchaseOrderDetailById(purchaseOrderVo.getId()));
            Supplier supplier = supplierMapper.getSupplierById(purchaseOrderVo.getSupplierId());
            String name = supplier.getName();
            BigDecimal totalAmount = purchaseOrderVo.getTotalAmount();

            supplierAmountMap.merge(
                    name,
                    totalAmount,
                    BigDecimal::add  // 相同供应商的金额相加
            );
//            xAxisSupplierNames.add(name);
//            yAxisOrderAmounts.add(totalAmount);
        }
        List<String> xAxisData = new ArrayList<>(supplierAmountMap.keySet()); // 横轴
        List<BigDecimal> yAxisData = new ArrayList<>(supplierAmountMap.values()); // 纵轴

        LocalDate currentDate = LocalDate.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String formattedDate = currentDate.format(formatter);
        String msg = "好的，" + formattedDate + "，采购入库订单数量为: " + list.size() + "，有其他需求可以随时告诉我哦！";

        purchaseOrderChartVo.setList(list);
        purchaseOrderChartVo.setChartType("column");
        purchaseOrderChartVo.setMsg(msg);
        purchaseOrderChartVo.setXAxisData(xAxisData);
        purchaseOrderChartVo.setYAxisData(yAxisData);


//        purchaseOrderChartVo.setXAxisData(xAxisSupplierNames);
//        purchaseOrderChartVo.setYAxisData(yAxisOrderAmounts);

        return purchaseOrderChartVo;

    }

    private TodaysPurchaseOrderReturnChartVo getTodaysPurchaseOrderReturn() {

        TodaysPurchaseOrderReturnChartVo todaysPurchaseOrderReturnChartVo = new TodaysPurchaseOrderReturnChartVo();
        List<TodaysPurchaseOrderReturnVo> list = purchaseReturnMapper.TodaysPurchaseOrderReturn();

//        List<String> xAxisSupplierNames = new ArrayList<>();
//        List<BigDecimal> yAxisOrderAmounts = new ArrayList<>();
        Map<String, BigDecimal> supplierAmountMap = new HashMap<>();
        for (TodaysPurchaseOrderReturnVo todaysPurchaseOrderReturnVo : list) {
            Supplier supplier = supplierMapper.getSupplierById(todaysPurchaseOrderReturnVo.getSupplierId());
            todaysPurchaseOrderReturnVo.setPurchaseOrderDetailList(purchaseReturnDetailMapper.getPurchaseReturnById(todaysPurchaseOrderReturnVo.getId()));

            String name = supplier.getName();
            BigDecimal totalAmount = todaysPurchaseOrderReturnVo.getTotalAmount();

            supplierAmountMap.merge(
                    name,
                    totalAmount,
                    BigDecimal::add  // 相同供应商的金额相加
            );
//            xAxisSupplierNames.add(name);
//            yAxisOrderAmounts.add(totalAmount);
        }

        List<String> xAxisData = new ArrayList<>(supplierAmountMap.keySet()); // 横轴
        List<BigDecimal> yAxisData = new ArrayList<>(supplierAmountMap.values()); // 纵轴
        todaysPurchaseOrderReturnChartVo.setList(list);
        todaysPurchaseOrderReturnChartVo.setChartType("column");

        LocalDate currentDate = LocalDate.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String formattedDate = currentDate.format(formatter);
        String msg = "好的，" + formattedDate + "，销售退货订单数量为: " + list.size() + "，有其他需求可以随时告诉我哦！";

        todaysPurchaseOrderReturnChartVo.setMsg(msg);
        todaysPurchaseOrderReturnChartVo.setXAxisData(xAxisData);
        todaysPurchaseOrderReturnChartVo.setYAxisData(yAxisData);
//        todaysPurchaseOrderReturnChartVo.setXAxisData(xAxisSupplierNames);
//        todaysPurchaseOrderReturnChartVo.setYAxisData(yAxisOrderAmounts);

        return todaysPurchaseOrderReturnChartVo;
    }

    private SaleOrderChartVo getSaleOrderByCustomerName(String customerName) {

        Customer customer = customerMapper.QueryCustomerName(customerName);


        SaleOrderChartVo chartVo = new SaleOrderChartVo();
        if (customer != null) {

            BigDecimal totalAmountSum = BigDecimal.ZERO;

            List<SaleOrderVo> list = saleOrderMapper.getSaleOrderByCustomerId(customer.getId());

//            List<String> xAxisSupplierNames = new ArrayList<>();
//            List<BigDecimal> yAxisOrderAmounts = new ArrayList<>();
            Map<String, BigDecimal> supplierAmountMap = new HashMap<>();

            for (SaleOrderVo saleOrderVo : list) {
                BigDecimal amount = Optional.ofNullable(saleOrderVo.getTotalAmount())
                        .orElse(BigDecimal.ZERO);

                // 累加到总金额
                totalAmountSum = totalAmountSum.add(amount);


                saleOrderVo.setSaleOrderDetailList(saleOrderDetailMapper.getSaleOrderDetailById(saleOrderVo.getId()));

                if (customer != null && customer.getName() != null) {
                    // 累加当前订单金额到供应商金额映射表
                    supplierAmountMap.merge(
                            customer.getName(),
                            amount, // 使用当前订单金额而非 totalAmountSum
                            BigDecimal::add
                    );
                }
//                xAxisSupplierNames.add(customer.getName());
//                yAxisOrderAmounts.add(totalAmountSum);
            }

            List<String> xAxisData = new ArrayList<>(supplierAmountMap.keySet()); // 横轴
            List<BigDecimal> yAxisData = new ArrayList<>(supplierAmountMap.values()); // 纵轴

            DecimalFormat currencyFormat = new DecimalFormat("￥####.00");

            String formattedTotalAmount="0";
            if(totalAmountSum!=null){
                formattedTotalAmount = currencyFormat.format(totalAmountSum);
            }



            String msg = "好的，" + customer.getName() +
                    "，订单数量为: " + list.size() +
                    "，金额为：" + formattedTotalAmount;

            chartVo.setList(list);
            chartVo.setChartType("column");
            chartVo.setMsg(msg);
            chartVo.setXAxisData(xAxisData);
            chartVo.setYAxisData(yAxisData);
//            chartVo.setXAxisData(xAxisSupplierNames);
//            chartVo.setYAxisData(yAxisOrderAmounts);


            return chartVo;
        } else {
            throw new IllegalArgumentException("客户不存在！");
        }


    }

    private SaleOrderChartVo getSaleOrderByUserName(String userName) {

        SysUserDTO user = sysUserMapper.getSysUserByName(userName);

        SaleOrderChartVo saleOrderChartVo = new SaleOrderChartVo();

        if (user != null) {
//            List<String> xAxisSupplierNames = new ArrayList<>();
//            List<BigDecimal> yAxisOrderAmounts = new ArrayList<>();
            Map<String, BigDecimal> supplierAmountMap = new HashMap<>();

            BigDecimal totalAmountSum = BigDecimal.ZERO;
            List<SaleOrderVo> list = saleOrderMapper.getSaleOrderBySalerId(user.getId());
            for (SaleOrderVo saleOrderVo : list) {
                BigDecimal amount = Optional.ofNullable(saleOrderVo.getTotalAmount())
                        .orElse(BigDecimal.ZERO);

                // 累加到总金额
                totalAmountSum = totalAmountSum.add(amount);
                saleOrderVo.setSaleOrderDetailList(saleOrderDetailMapper.getSaleOrderDetailById(saleOrderVo.getId()));

                if (user != null && user.getName() != null) {
                    // 累加当前订单金额到供应商金额映射表
                    supplierAmountMap.merge(
                            user.getName(),
                            amount, // 使用当前订单金额而非 totalAmountSum
                            BigDecimal::add
                    );
                }
//                xAxisSupplierNames.add(user.getName());
//                yAxisOrderAmounts.add(totalAmountSum);
            }

            List<String> xAxisData = new ArrayList<>(supplierAmountMap.keySet()); // 横轴
            List<BigDecimal> yAxisData = new ArrayList<>(supplierAmountMap.values()); // 纵轴

            DecimalFormat currencyFormat = new DecimalFormat("￥####.00");
            String formattedTotalAmount="0";
            if(totalAmountSum!=null){
                formattedTotalAmount = currencyFormat.format(totalAmountSum);
            }

            String msg = "好的，" + user.getName() +
                    "，销售订单数量为: " + list.size() +
                    "，金额为：" + formattedTotalAmount;

            saleOrderChartVo.setList(list);
            saleOrderChartVo.setChartType("column");
            saleOrderChartVo.setMsg(msg);

            saleOrderChartVo.setXAxisData(xAxisData);
            saleOrderChartVo.setYAxisData(yAxisData);
//            saleOrderChartVo.setXAxisData(xAxisSupplierNames);
//            saleOrderChartVo.setYAxisData(yAxisOrderAmounts);


            return saleOrderChartVo;
        } else {
            throw new IllegalArgumentException("销售员不存在！");
        }
    }

    private SaleOrderChartVo selectSaleOrdersByDate(String targetDate) {

        SaleOrderChartVo saleOrderChartVo = new SaleOrderChartVo();
        List<SaleOrderVo> list = saleOrderMapper.selectSaleOrdersByDate(targetDate);
        BigDecimal totalAmountSum = BigDecimal.ZERO;

//        List<String> xAxisSupplierNames = new ArrayList<>();
//        List<BigDecimal> yAxisOrderAmounts = new ArrayList<>();
        Map<String, BigDecimal> supplierAmountMap = new HashMap<>();
        for (SaleOrderVo saleOrderVo : list) {
            BigDecimal amount = Optional.ofNullable(saleOrderVo.getTotalAmount())
                    .orElse(BigDecimal.ZERO);

            // 累加到总金额
            totalAmountSum = totalAmountSum.add(amount);
            saleOrderVo.setSaleOrderDetailList(saleOrderDetailMapper.getSaleOrderDetailById(saleOrderVo.getId()));

            if (saleOrderVo.getSalerId() != null) {
                SysUserDTO user = sysUserMapper.getSysUserById(saleOrderVo.getSalerId());

                if (user != null && user.getName() != null) {
                    // 累加当前订单金额到供应商金额映射表
                    supplierAmountMap.merge(
                            user.getName(),
                            amount, // 使用当前订单金额而非 totalAmountSum
                            BigDecimal::add
                    );
                }
//               xAxisSupplierNames.add(user.getName());
//               yAxisOrderAmounts.add(totalAmountSum);
            }


        }

        List<String> xAxisData = new ArrayList<>(supplierAmountMap.keySet()); // 横轴
        List<BigDecimal> yAxisData = new ArrayList<>(supplierAmountMap.values()); // 纵轴

        DecimalFormat currencyFormat = new DecimalFormat("￥####.00");
        String formattedTotalAmount="0";
        if(totalAmountSum!=null){
            formattedTotalAmount = currencyFormat.format(totalAmountSum);
        }
        String msg = "好的，" + targetDate +
                "，销售订单数量为: " + list.size() +
                "，金额为：" + formattedTotalAmount;

        saleOrderChartVo.setList(list);
        saleOrderChartVo.setChartType("column");
        saleOrderChartVo.setMsg(msg);
//        saleOrderChartVo.setXAxisData(xAxisSupplierNames);
//        saleOrderChartVo.setYAxisData(yAxisOrderAmounts);
        saleOrderChartVo.setXAxisData(xAxisData);
        saleOrderChartVo.setYAxisData(yAxisData);

        return saleOrderChartVo;
    }

    private PurchaseOrderChartVo selectPurchaseOrdersByDate(String targetDate) {

        PurchaseOrderChartVo purchaseOrderChartVo = new PurchaseOrderChartVo();
        List<PurchaseOrderVo> list = purchaseOrderMapper.selectPruchaseOrdersByDate(targetDate);
        BigDecimal totalAmountSum = BigDecimal.ZERO;

        Map<String, BigDecimal> supplierAmountMap = new HashMap<>();
        for (PurchaseOrderVo purchaseOrderVo : list) {

            BigDecimal amount = Optional.ofNullable(purchaseOrderVo.getTotalAmount())
                    .orElse(BigDecimal.ZERO);

            // 累加到总金额
            totalAmountSum = totalAmountSum.add(amount);
            purchaseOrderVo.setPurchaseOrderDetailList(purchaseOrderDetailMapper.getPurchaseOrderDetailById(purchaseOrderVo.getId()));

            Supplier supplier = supplierMapper.getSupplierById(purchaseOrderVo.getSupplierId());
            if (supplier != null && supplier.getName() != null) {
                // 累加当前订单金额到供应商金额映射表
                supplierAmountMap.merge(
                        supplier.getName(),
                        amount, // 使用当前订单金额而非 totalAmountSum
                        BigDecimal::add
                );
            }
        }


        List<String> xAxisData = new ArrayList<>(supplierAmountMap.keySet()); // 横轴
        List<BigDecimal> yAxisData = new ArrayList<>(supplierAmountMap.values()); // 纵轴

        DecimalFormat currencyFormat = new DecimalFormat("￥####.00");
        String formattedTotalAmount="0";
        if(totalAmountSum!=null){
            formattedTotalAmount = currencyFormat.format(totalAmountSum);
        }

        String msg = "好的，" + targetDate +
                "，采购订单数量为: " + list.size() +
                "，金额为：" + formattedTotalAmount;

        purchaseOrderChartVo.setMsg(msg);
        purchaseOrderChartVo.setList(list);
        purchaseOrderChartVo.setChartType("column");

        purchaseOrderChartVo.setXAxisData(xAxisData);
        purchaseOrderChartVo.setYAxisData(yAxisData);
        return purchaseOrderChartVo;
    }

    public ProductStockLogChartVo getProductStockLogs(String productName) {

        ProductStockLogChartVo productStockLogChartVo = new ProductStockLogChartVo();
        List<Product> products = productMapper.QueryProductByName(productName);
        List<ProductStockLogVo> allLogs = new ArrayList<>();

        for (Product product : products) {
            List<ProductStockLogVo> logs = productStockLogMapper.getProductStockLogByProductId(product.getId());
            if (logs != null && !logs.isEmpty()) {
                logs.forEach(log -> log.setProductName(product.getName())); // 设置商品名
                allLogs.addAll(logs);
            }
        }

        Collections.sort(allLogs, Comparator.comparing(ProductStockLogVo::getCreateTime));

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-M-d");
        StringBuilder msgBuilder = new StringBuilder();

        if (allLogs.isEmpty()) {
            msgBuilder.append("暂无库存变动记录");
        } else {
            // 获取截止日期（最后一条日志的日期）
            Date latestDate = allLogs.get(allLogs.size() - 1).getCreateTime();
            msgBuilder.append("截止（").append(sdf.format(latestDate)).append("）库存变动记录如下，");

            // 拼接每条记录
            List<String> entries = new ArrayList<>();
            for (ProductStockLogVo log : allLogs) {
                String dateStr = sdf.format(log.getCreateTime());
                String action = log.getStockNum() > 0 ? "入库" : "出库";
                // 重点修改：商品名用括号包裹
                String entry = String.format("（%s）%s%s%d",
                        log.getProductName(), dateStr, action, Math.abs(log.getStockNum()));
                entries.add(entry);
            }
            msgBuilder.append(String.join("，", entries)).append("。");
        }

        productStockLogChartVo.setList(allLogs);
        productStockLogChartVo.setChartType("null");
        productStockLogChartVo.setMsg(msgBuilder.toString());

        return productStockLogChartVo;
    }

    private SaleOutSheetChartVo selectSaleOutSheetToDays() {

        SaleOutSheetChartVo saleOutSheetChartVo = new SaleOutSheetChartVo();
        List<SaleOutSheetVo> list = saleOutSheetMapper.getTodysSaleOutSheet();

        for (SaleOutSheetVo saleOutSheetVo : list) {
            saleOutSheetVo.setSaleOutSheetDetailList(saleOutSheetDetailMapper.getSaleOutSheetDetailById(saleOutSheetVo.getId()));
        }

        LocalDate currentDate = LocalDate.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String formattedDate = currentDate.format(formatter);
        String msg = "好的，" + formattedDate + "，销售出库订单数量为: " + list.size() + "，有其他需求可以随时告诉我哦！";

        saleOutSheetChartVo.setList(list);
        saleOutSheetChartVo.setChartType("null");
        saleOutSheetChartVo.setMsg(msg);

        return saleOutSheetChartVo;
    }

    private CheckOrderAmountCountVo checkOrderAmountCount() {

        CheckOrderAmountCountVo vo = new CheckOrderAmountCountVo();
        LocalDate currentDate = LocalDate.now();
        DateTimeFormatter dateFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        DecimalFormat currencyFormat = new DecimalFormat("￥####.00");

        // 获取BigDecimal数据（处理null值）
        BigDecimal totalAmount = saleOrderMapper.CheckOrderAmountCount() != null ?
                saleOrderMapper.CheckOrderAmountCount() : BigDecimal.ZERO;
        BigDecimal purchasePrice = saleOrderMapper.CheckOrderPurchasePrice() != null ?
                saleOrderMapper.CheckOrderPurchasePrice() : BigDecimal.ZERO;

        // 计算总和（BigDecimal运算）
        BigDecimal total = totalAmount.add(purchasePrice);

        // 计算百分比（带精度控制）
        DecimalFormat percentFormat = new DecimalFormat("0.00%");
        String revenuePercent = "0.00%";
        String costPercent = "0.00%";
        if (total.compareTo(BigDecimal.ZERO) != 0) {
            revenuePercent = percentFormat.format(
                    totalAmount.divide(total, 4, RoundingMode.HALF_UP)
            );
            costPercent = percentFormat.format(
                    purchasePrice.divide(total, 4, RoundingMode.HALF_UP)
            );
        }

        // 构建饼图数据（处理数值类型转换）
        List<PieChartData> pieData = new ArrayList<>();
        pieData.add(new PieChartData(
                "营收账款",
                totalAmount.setScale(0, RoundingMode.HALF_UP).intValueExact(),
                revenuePercent
        ));
        pieData.add(new PieChartData(
                "成本",
                purchasePrice.setScale(0, RoundingMode.HALF_UP).intValueExact(),
                costPercent
        ));

        // 构建消息（BigDecimal格式化）
        String msg = String.format("好的，截止%s，营收账款: %s，成本: %s，营收占比%s。有需求随时告诉我！",
                currentDate.format(dateFormatter),
                currencyFormat.format(totalAmount),
                currencyFormat.format(purchasePrice),
                revenuePercent);

        // 设置VO对象
        vo.setAmountCount(totalAmount);  // 保持与现有字段兼容
        vo.setChartType("ring");
        vo.setMsg(msg);
        vo.setPieChartData(pieData);

        return vo;
    }

    private SettleCheckAccountsPayableVo settleCheckAccountsPayable() {

        SettleCheckAccountsPayableVo settleCheckAccountsPayableVo = new SettleCheckAccountsPayableVo();

        LocalDate currentDate = LocalDate.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String formattedDate = currentDate.format(formatter);

        DecimalFormat currencyFormat = new DecimalFormat("￥####.00");

        String formattedTotalAmount="0";

        if(saleOrderMapper.CheckOrderAmountCount()!=null){
            formattedTotalAmount = currencyFormat.format(settleCheckSheetMapper.settleCheckAccountsPayable());
        }


        String msg = "好的，截止" + formattedDate + "，应付账款为: " + formattedTotalAmount + "，有其他需求可以随时告诉我哦！";

        settleCheckAccountsPayableVo.setTotalAmount(settleCheckSheetMapper.settleCheckAccountsPayable());
        settleCheckAccountsPayableVo.setChartType("null");
        settleCheckAccountsPayableVo.setMsg(msg);
        return settleCheckAccountsPayableVo;
    }

    private SettleSheetPaymentOfFeesVo settleSheetPaymentOfFees() {

        SettleSheetPaymentOfFeesVo settleSheetPaymentOfFeesVo = new SettleSheetPaymentOfFeesVo();

        LocalDate currentDate = LocalDate.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String formattedDate = currentDate.format(formatter);

        DecimalFormat currencyFormat = new DecimalFormat("￥####.00");

        String formattedTotalAmount="0";

        if(saleOrderMapper.CheckOrderAmountCount()!=null){
            formattedTotalAmount = currencyFormat.format(settleSheetMapper.settleSheetPaymentOfFees());
        }

        String msg = "好的，截止" + formattedDate + "，费用支付为: " + formattedTotalAmount + "，有其他需求可以随时告诉我哦！";

        settleSheetPaymentOfFeesVo.setTotalAmount(settleSheetMapper.settleSheetPaymentOfFees());
        settleSheetPaymentOfFeesVo.setChartType("null");
        settleSheetPaymentOfFeesVo.setMsg(msg);
        return settleSheetPaymentOfFeesVo;
    }

    private BigDecimal safeDivide(BigDecimal dividend, BigDecimal divisor, int scale) {
        return divisor.compareTo(BigDecimal.ZERO) != 0 ?
                dividend.divide(divisor, scale, RoundingMode.HALF_UP) :
                BigDecimal.ZERO;
    }

    private SaleOrderTotalProfitVo saleOrderTotalProfit() {

        SaleOrderTotalProfitVo vo = new SaleOrderTotalProfitVo();
        List<SaleOrder> orders = saleOrderMapper.getTotalProfit();

        // 初始化累计值
        BigDecimal totalRevenue = BigDecimal.ZERO;
        BigDecimal totalCost = BigDecimal.ZERO;

        // 累计计算（使用BigDecimal精确计算）
        for (SaleOrder order : orders) {
            totalRevenue = totalRevenue.add(
                    Optional.ofNullable(order.getTotalAmount()).orElse(BigDecimal.ZERO)
            );
            totalCost = totalCost.add(
                    Optional.ofNullable(order.getTotalPurchasePrice()).orElse(BigDecimal.ZERO)
            );
        }

        // 计算利润（营收 - 成本）
        BigDecimal totalProfit = totalRevenue.subtract(totalCost);

        // 计算利润占比（带安全校验）
        DecimalFormat percentFormat = new DecimalFormat("0.00%");
        BigDecimal profitRatio = totalRevenue.compareTo(BigDecimal.ZERO) != 0 ?
                totalProfit.divide(totalRevenue, 4, RoundingMode.HALF_UP) :
                BigDecimal.ZERO;

        // 构建饼图数据（包含两种维度）
        List<PieChartData> pieData = new ArrayList<>();

        // 维度1：利润/成本结构
        try {
            int profitValue = totalProfit.setScale(0, RoundingMode.HALF_UP).intValueExact();
            pieData.add(new PieChartData("利润", profitValue, percentFormat.format(profitRatio)));
        } catch (ArithmeticException e) {
            // 如果发生溢出或无法精确转换，记录日志并设置一个默认值
            pieData.add(new PieChartData("利润", Integer.MAX_VALUE, percentFormat.format(profitRatio)));
        }

        try {
            int costValue = totalCost.setScale(0, RoundingMode.HALF_UP).intValueExact();
            pieData.add(new PieChartData("成本", costValue, percentFormat.format(totalCost.divide(totalRevenue, 4, RoundingMode.HALF_UP))));
        } catch (ArithmeticException e) {
            pieData.add(new PieChartData("成本", Integer.MAX_VALUE, percentFormat.format(totalCost.divide(totalRevenue, 4, RoundingMode.HALF_UP))));
        }

        // 维度2：利润率分布（可选）
//        BigDecimal profitMargin = totalRevenue.compareTo(BigDecimal.ZERO) != 0 ?
//                totalProfit.divide(totalRevenue, 4, RoundingMode.HALF_UP) :
//                BigDecimal.ZERO;
//
//        BigDecimal profitMarginPercent = profitMargin.multiply(BigDecimal.valueOf(100))
//                .setScale(0, RoundingMode.HALF_UP); // 四舍五入到整数
//
//        BigDecimal otherPercent = BigDecimal.valueOf(100)
//                .subtract(profitMarginPercent)
//                .setScale(0, RoundingMode.HALF_UP);
//
//        List<PieChartData> marginData = Arrays.asList(
//                new PieChartData("利润率", profitMarginPercent.intValueExact(), percentFormat.format(profitMargin)),
//                new PieChartData("其他", otherPercent.intValueExact(), "")
//        );

        // 消息构建
        String msg = String.format("截止%s，总营收%s，总成本%s，总利润%s，利润占比%s。",
                LocalDate.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")),
                new DecimalFormat("￥#,##0.00").format(totalRevenue),
                new DecimalFormat("￥#,##0.00").format(totalCost),
                new DecimalFormat("￥#,##0.00").format(totalProfit),
                percentFormat.format(profitRatio)
        );

        // VO对象装配
        vo.setRevenue(totalRevenue);
        vo.setCost(totalCost);
        vo.setTotalProfit(totalProfit);
        vo.setChartType("ring");
        vo.setMsg(msg);
        vo.setPieChartData(pieData);  // 核心业务数据
      //  vo.setMarginChartData(marginData); // 扩展维度数据

        return vo;
    }


    private CheckTodysSalesAmountVo getMonthSalesAmount() {

        CheckTodysSalesAmountVo checkTodysSalesAmountVo = new CheckTodysSalesAmountVo();
        checkTodysSalesAmountVo.setChartType("column");
        checkTodysSalesAmountVo.setSalesAmount(saleOrderMapper.CheckMonthOrderAmount());

        List<SaleOrderVo> list = saleOrderMapper.CheckMonthSaleOrderLits();

        Map<String, BigDecimal> supplierAmountMap = new HashMap<>();
        for (SaleOrderVo saleOrderVo : list) {
            if (saleOrderVo != null) {
                if (saleOrderVo.getSalerId() != null) {
                    SysUserDTO sysUserDTO = sysUserMapper.getSysUserById(saleOrderVo.getSalerId());
                    String name = sysUserDTO.getName();
                    saleOrderVo.setSalerName(name);
                    BigDecimal totalAmount = saleOrderVo.getTotalAmount();

                    supplierAmountMap.merge(
                            name,
                            totalAmount,
                            BigDecimal::add  // 相同供应商的金额相加
                    );
                }

            }

        }

        List<String> xAxisData = new ArrayList<>(supplierAmountMap.keySet()); // 横轴
        List<BigDecimal> yAxisData = new ArrayList<>(supplierAmountMap.values()); // 纵轴

        DecimalFormat currencyFormat = new DecimalFormat("￥####.00");

        String formattedTotalAmount=null;
        if(saleOrderMapper.CheckMonthOrderAmount()!=null){
            formattedTotalAmount = currencyFormat.format(saleOrderMapper.CheckMonthOrderAmount());
        }else{
            formattedTotalAmount="0";
        }


        String msg = "好的，" +  "本月销售订单金额为: " + formattedTotalAmount + "，有其他需求可以随时告诉我哦！";
        checkTodysSalesAmountVo.setMsg(msg);

        checkTodysSalesAmountVo.setXAxisData(xAxisData);
        checkTodysSalesAmountVo.setYAxisData(yAxisData);

        return checkTodysSalesAmountVo;
    }

    private CheckTodysSalesAmountVo getYearSalesAmount() {

        CheckTodysSalesAmountVo checkTodysSalesAmountVo = new CheckTodysSalesAmountVo();
        checkTodysSalesAmountVo.setChartType("column");
        checkTodysSalesAmountVo.setSalesAmount(saleOrderMapper.CheckYearOrderAmount());

        List<SaleOrderVo> list = saleOrderMapper.CheckYearSaleOrderLits();

        Map<String, BigDecimal> supplierAmountMap = new HashMap<>();
        for (SaleOrderVo saleOrderVo : list) {
            if (saleOrderVo != null) {
                if (saleOrderVo.getSalerId() != null) {
                    SysUserDTO sysUserDTO = sysUserMapper.getSysUserById(saleOrderVo.getSalerId());
                    String name = sysUserDTO.getName();
                    saleOrderVo.setSalerName(name);
                    BigDecimal totalAmount = saleOrderVo.getTotalAmount();

                    supplierAmountMap.merge(
                            name,
                            totalAmount,
                            BigDecimal::add  // 相同供应商的金额相加
                    );
                }

            }

        }

        List<String> xAxisData = new ArrayList<>(supplierAmountMap.keySet()); // 横轴
        List<BigDecimal> yAxisData = new ArrayList<>(supplierAmountMap.values()); // 纵轴

        DecimalFormat currencyFormat = new DecimalFormat("￥####.00");

        String formattedTotalAmount=null;
        if(saleOrderMapper.CheckYearOrderAmount()!=null){
            formattedTotalAmount = currencyFormat.format(saleOrderMapper.CheckYearOrderAmount());
        }else{
            formattedTotalAmount="0";
        }


        String msg = "好的，" +  "本年销售订单金额为: " + formattedTotalAmount + "，有其他需求可以随时告诉我哦！";
        checkTodysSalesAmountVo.setMsg(msg);

        checkTodysSalesAmountVo.setXAxisData(xAxisData);
        checkTodysSalesAmountVo.setYAxisData(yAxisData);

        return checkTodysSalesAmountVo;
    }

    private CheckTodaysPurchaseAmountVo getMonthPurchaseAmount() {

        CheckTodaysPurchaseAmountVo checkTodaysPurchaseAmountVo = new CheckTodaysPurchaseAmountVo();
        checkTodaysPurchaseAmountVo.setChartType("column");
        BigDecimal amount = purchaseOrderMapper.CheckMonthPurchaseAmount();


        checkTodaysPurchaseAmountVo.setPurchaseAmount(amount);

        List<PurchaseOrderVo> list = purchaseOrderMapper.MonthPurchaseOrder();
        Map<String, BigDecimal> supplierAmountMap = new HashMap<>();

        for (PurchaseOrderVo purchaseOrderVo : list) {
            Supplier supplier = supplierMapper.getSupplierById(purchaseOrderVo.getSupplierId());
            String supplierName = supplier.getName();
            purchaseOrderVo.setSupplierName(supplierName);
            BigDecimal totalAmount = purchaseOrderVo.getTotalAmount();

            supplierAmountMap.merge(
                    supplierName,
                    totalAmount,
                    BigDecimal::add  // 相同供应商的金额相加
            );

        }

        List<String> xAxisData = new ArrayList<>(supplierAmountMap.keySet()); // 横轴
        List<BigDecimal> yAxisData = new ArrayList<>(supplierAmountMap.values()); // 纵轴

        DecimalFormat currencyFormat = new DecimalFormat("￥####.00");

        String formattedTotalAmount=null;
        if(purchaseOrderMapper.CheckMonthPurchaseAmount()!=null){
            formattedTotalAmount = currencyFormat.format(amount);
        }else{
            formattedTotalAmount="0";
        }

        String msg = "好的，本月采购订单金额为: " + formattedTotalAmount + "，有其他需求可以随时告诉我哦！";

        checkTodaysPurchaseAmountVo.setMsg(msg);
        checkTodaysPurchaseAmountVo.setXAxisData(xAxisData);
        checkTodaysPurchaseAmountVo.setYAxisData(yAxisData);

        return checkTodaysPurchaseAmountVo;
    }

    private CheckTodaysPurchaseAmountVo getYearPurchaseAmount() {

        CheckTodaysPurchaseAmountVo checkTodaysPurchaseAmountVo = new CheckTodaysPurchaseAmountVo();
        checkTodaysPurchaseAmountVo.setChartType("column");
        BigDecimal amount = purchaseOrderMapper.CheckYearPurchaseAmount();


        checkTodaysPurchaseAmountVo.setPurchaseAmount(amount);

        List<PurchaseOrderVo> list = purchaseOrderMapper.YearPurchaseOrder();
        Map<String, BigDecimal> supplierAmountMap = new HashMap<>();

        for (PurchaseOrderVo purchaseOrderVo : list) {
            Supplier supplier = supplierMapper.getSupplierById(purchaseOrderVo.getSupplierId());
            String supplierName = supplier.getName();
            purchaseOrderVo.setSupplierName(supplierName);
            BigDecimal totalAmount = purchaseOrderVo.getTotalAmount();

            supplierAmountMap.merge(
                    supplierName,
                    totalAmount,
                    BigDecimal::add  // 相同供应商的金额相加
            );

        }

        List<String> xAxisData = new ArrayList<>(supplierAmountMap.keySet()); // 横轴
        List<BigDecimal> yAxisData = new ArrayList<>(supplierAmountMap.values()); // 纵轴

        DecimalFormat currencyFormat = new DecimalFormat("￥####.00");

        String formattedTotalAmount=null;
        if(purchaseOrderMapper.CheckYearPurchaseAmount()!=null){
            formattedTotalAmount = currencyFormat.format(amount);
        }else{
            formattedTotalAmount="0";
        }

        String msg = "好的，本年采购订单金额为: " + formattedTotalAmount + "，有其他需求可以随时告诉我哦！";

        checkTodaysPurchaseAmountVo.setMsg(msg);
        checkTodaysPurchaseAmountVo.setXAxisData(xAxisData);
        checkTodaysPurchaseAmountVo.setYAxisData(yAxisData);

        return checkTodaysPurchaseAmountVo;
    }

}
