package com.soft.utils;

import com.soft.entitys.QueryParams;
import com.soft.enums.QueryType;

import javax.management.Query;
import java.time.LocalDate;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class VoiceQueryParser {

    public QueryParams parse(String voiceText) {

        QueryParams params = new QueryParams();


        // 解析查询类型和参数

        if (voiceText.contains("今日采购订单") || voiceText.contains("今天采购订单") ) {
            params.setType(QueryType.TODAY_PURCHASE_AMOUNT);
        }
        else if (voiceText.contains("今日销售订单") || voiceText.contains("今天销售订单")) {
            params.setType(QueryType.TODAY_SALES_AMOUNT);
        }
        else if(voiceText.contains("日") &&voiceText.contains("销售订单")   || voiceText.contains("号") &&voiceText.contains("销售订单")){
            params.setType(QueryType.SALES_ORDERS_BY_DATE);
            extractDateFromText(voiceText,params);
        }
        else if(voiceText.contains("日") &&voiceText.contains("采购订单")   || voiceText.contains("号") &&voiceText.contains("采购订单")){
            params.setType(QueryType.PURCHASE_ORDERS_BY_DATE);
            extractDateFromText(voiceText,params);
        }
        else if(voiceText.contains("本月销售订单")){
            params.setType(QueryType.MONTH_SALE_AMOUNT);
            extractDateFromText(voiceText,params);
        }
        else if(voiceText.contains("本月采购订单")){
            params.setType(QueryType.MONTH_PURCHASE_AMOUNT);
            extractDateFromText(voiceText,params);
        }
        else if(voiceText.contains("本年销售订单")){
            params.setType(QueryType.YEAR_SALE_AMOUNT);
            extractDateFromText(voiceText,params);
        }
        else if(voiceText.contains("本年采购订单")){
            params.setType(QueryType.YEAR_PURCHASE_AMOUNT);
            extractDateFromText(voiceText,params);
        }
        else if (voiceText.contains("库存") && voiceText.contains("多少") ||voiceText.contains("库存") && voiceText.contains("剩余")) {
            params.setType(QueryType.PRODUCT_INVENTORY);
            extractProductName(voiceText, params);
        }
        else if(voiceText.contains("采购入库订单")&& voiceText.contains("今日") || voiceText.contains("采购入库订单") &&voiceText.contains("今天")){
            params.setType(QueryType.TODAY_PURCHASE_ORDERS);
        }
        else if(voiceText.contains("采购退货订单") && voiceText.contains("今日") || voiceText.contains("采购退货订单") && voiceText.contains("今天")){
            params.setType(QueryType.TODAY_PURCHASE_RETURN);
        }
        else if(voiceText.contains("的订单")){
            params.setType(QueryType.CUSTOMER_NAME_SEARCH);
            extractCustomer(voiceText,params);
        }
        else if(voiceText.contains("销售订单") ||voiceText.contains("销售记录") ){
            params.setType(QueryType.USER_NAME_ORDERS_SEARCH);
            extractSalesperson(voiceText,params);
        }
        else if(voiceText.contains("出入库记录")||voiceText.contains("出入库流水记录")){
            params.setType(QueryType.PRODUCT_STOCK_LOG_BY_NAME);
            extractProductStock(voiceText,params);
        }
        else if(voiceText.contains("今日销售出库订单")||voiceText.contains("今天销售出库订单")){
            params.setType(QueryType.TODAY_OUTBOUND_ORDERS);
        }
        else if(voiceText.contains("营收账款") || voiceText.contains("合同金额")){
            params.setType(QueryType.REVENUE_COUNT);
        }
        else if(voiceText.contains("应付账款")){
            params.setType(QueryType.ACCOUNTS_PAYABLE);
        }
        else if(voiceText.contains("费用支付")){
            params.setType(QueryType.PAYMENT_TOF_FEES);
        }
        else if(voiceText.contains("利润总和")){
            params.setType(QueryType.TOTAL_PROFIT);
        }

        return params;
    }

    private void extractProductName(String text, QueryParams params) {
        // 使用正则表达式提取商品名
        Pattern pattern = Pattern.compile("(?:查询|查看|查找|查)(.+?)(?:的?)(?:库存|存货|余量|数量|剩下|剩余)");
        Matcher matcher = pattern.matcher(text);
        if (matcher.find()) {
            params.setProductName(matcher.group(1).trim());
        } else {
            // 尝试其他模式
            pattern = Pattern.compile("(?:库存|存货|余量|数量|剩下|剩余)(.+?)(?:的?)(?:有多少|是多少|多少|几多)");
            matcher = pattern.matcher(text);
            if (matcher.find()) {
                params.setProductName(matcher.group(1).trim());
            }
        }
    }

    private void extractProductStock(String text, QueryParams params) {
        // 使用正则表达式提取销售人员
        Pattern pattern = Pattern.compile("(?:查询|查看|查找|查)(.+?)(?:的?)(?:出入库记录|出入库流水记录)");
        Matcher matcher = pattern.matcher(text);
        if (matcher.find()) {
            params.setProductName(matcher.group(1).trim());
        }
    }

    private void extractSalesperson(String text, QueryParams params) {
        // 使用正则表达式提取销售人员
        Pattern pattern = Pattern.compile("(?:查询|查看|查找|查)(.+?)(?:的?)(?:销售记录|销售订单)");
        Matcher matcher = pattern.matcher(text);
        if (matcher.find()) {
            params.setUserName(matcher.group(1).trim());
        }
    }

    private void extractCustomer(String text, QueryParams params) {
        // 更灵活的正则表达式，支持多种表达方式
        Pattern pattern = Pattern.compile("(?:查询|查看|查找|查)(.+?)(?:的?)(?:订单)");
        Matcher matcher = pattern.matcher(text);
        if (matcher.find()) {
            params.setCustomerName(matcher.group(1).trim());
        }

    }

    private void extractDateFromText(String text, QueryParams param) {
        // 更灵活的正则表达式，处理各种情况
        Pattern datePattern = Pattern.compile(
                "(?:查询|查看|查找)?\\s*" +  // 可选前缀
                        "(\\d{1,2}\\s*月\\s*\\d{1,2}\\s*[日号]|" +  // 4月28号
                        "\\d{4}\\s*年\\s*\\d{1,2}\\s*月\\s*\\d{1,2}\\s*[日号])"  // 2024年4月28日
        );

        Matcher matcher = datePattern.matcher(text.replaceAll("\\s+", "")); // 去除所有空格
        if (matcher.find()) {
            String matchedDate = matcher.group(1).trim();
            String standardDate = DateParserUtil.parseChineseDate(matchedDate);
            param.setQueryDate(standardDate);
        }
    }


}
